from briefcase.bootstraps.base import BaseGuiBootstrap


class TogaGuiBootstrap(BaseGuiBootstrap):
    def app_source(self):
        return '''\
import toga
from toga.style import Pack
from toga.style.pack import COLUMN, ROW


class {{ cookiecutter.class_name }}(toga.App):
    def startup(self):
        """Construct and show the Toga application.

        Usually, you would add your application to a main content box.
        We then create a main window (with a name matching the app), and
        show the main window.
        """
        main_box = toga.Box()

        self.main_window = toga.MainWindow(title=self.formal_name)
        self.main_window.content = main_box
        self.main_window.show()


def main():
    return {{ cookiecutter.class_name }}()
'''

    def app_start_source(self):
        return """\
from {{ cookiecutter.module_name }}.app import main

if __name__ == "__main__":
    main().main_loop()
"""

    def pyproject_table_briefcase_app_extra_content(self):
        return """
requires = [
]
test_requires = [
{% if cookiecutter.test_framework == "pytest" %}
    "pytest",
{% endif %}
]
"""

    def pyproject_table_macOS(self):
        return """\
universal_build = true
requires = [
    "toga-cocoa~=0.4.5",
    "std-nslog~=1.0.0",
]
"""

    def pyproject_table_linux(self):
        return """\
requires = [
    "toga-gtk~=0.4.5",
]
"""

    def pyproject_table_linux_system_debian(self):
        return """\
system_requires = [
    # Needed to compile pycairo wheel
    "libcairo2-dev",
    # Needed to compile PyGObject wheel
    "libgirepository1.0-dev",
]

system_runtime_requires = [
    # Needed to provide GTK and its GI bindings
    "gir1.2-gtk-3.0",
    "libgirepository-1.0-1",
    # Dependencies that GTK looks for at runtime
    "libcanberra-gtk3-module",
    # Needed to provide WebKit2 at runtime
    # Note: Debian 11 and Ubuntu 20.04 require gir1.2-webkit2-4.0 instead
    # "gir1.2-webkit2-4.1",
]
"""

    def pyproject_table_linux_system_rhel(self):
        return """\
system_requires = [
    # Needed to compile pycairo wheel
    "cairo-gobject-devel",
    # Needed to compile PyGObject wheel
    "gobject-introspection-devel",
]

system_runtime_requires = [
    # Needed to support Python bindings to GTK
    "gobject-introspection",
    # Needed to provide GTK
    "gtk3",
    # Dependencies that GTK looks for at runtime
    "libcanberra-gtk3",
    # Needed to provide WebKit2 at runtime
    # "webkit2gtk3",
]
"""

    def pyproject_table_linux_system_suse(self):
        return """\
system_requires = [
    # Needed to compile pycairo wheel
    "cairo-devel",
    # Needed to compile PyGObject wheel
    "gobject-introspection-devel",
]

system_runtime_requires = [
    # Needed to provide GTK
    "gtk3",
    # Needed to support Python bindings to GTK
    "gobject-introspection", "typelib(Gtk) = 3.0",
    # Dependencies that GTK looks for at runtime
    "libcanberra-gtk3-module",
    # Needed to provide WebKit2 at runtime
    # "libwebkit2gtk3", "typelib(WebKit2)",
]
"""

    def pyproject_table_linux_system_arch(self):
        return """\
system_requires = [
    # Needed to compile pycairo wheel
    "cairo",
    # Needed to compile PyGObject wheel
    "gobject-introspection",
    # Runtime dependencies that need to exist so that the
    # Arch package passes final validation.
    # Needed to provide GTK
    "gtk3",
    # Dependencies that GTK looks for at runtime
    "libcanberra",
    # Needed to provide WebKit2
    # "webkit2gtk",
]

system_runtime_requires = [
    # Needed to provide GTK
    "gtk3",
    # Needed to provide PyGObject bindings
    "gobject-introspection-runtime",
    # Dependencies that GTK looks for at runtime
    "libcanberra",
    # Needed to provide WebKit2 at runtime
    # "webkit2gtk",
]
"""

    def pyproject_table_linux_appimage(self):
        return """\
manylinux = "manylinux_2_28"

system_requires = [
    # Needed to compile pycairo wheel
    "cairo-gobject-devel",
    # Needed to compile PyGObject wheel
    "gobject-introspection-devel",
    # Needed to provide GTK
    "gtk3-devel",
    # Dependencies that GTK looks for at runtime, that need to be
    # in the build environment to be picked up by linuxdeploy
    "libcanberra-gtk3",
    "PackageKit-gtk3-module",
    "gvfs-client",
]

linuxdeploy_plugins = [
    "DEPLOY_GTK_VERSION=3 gtk",
]
"""

    def pyproject_table_linux_flatpak(self):
        return """\
flatpak_runtime = "org.gnome.Platform"
flatpak_runtime_version = "45"
flatpak_sdk = "org.gnome.Sdk"
"""

    def pyproject_table_windows(self):
        return """\
requires = [
    "toga-winforms~=0.4.5",
]
"""

    def pyproject_table_iOS(self):
        return """\
requires = [
    "toga-iOS~=0.4.5",
    "std-nslog~=1.0.0",
]
"""

    def pyproject_table_android(self):
        return """\
requires = [
    "toga-android~=0.4.5",
]

base_theme = "Theme.MaterialComponents.Light.DarkActionBar"

build_gradle_dependencies = [
    "com.google.android.material:material:1.11.0",
    # Needed for DetailedList
    # "androidx.swiperefreshlayout:swiperefreshlayout:1.1.0",
    # Needed for MapView
    # "org.osmdroid:osmdroid-android:6.1.0",
]
"""

    def pyproject_table_web(self):
        return """\
requires = [
    "toga-web~=0.4.5",
]
style_framework = "Shoelace v2.3"
"""



import shutil
from unittest.mock import MagicMock

import pytest

from briefcase.integrations.base import ToolCache
from briefcase.integrations.file import File
from briefcase.integrations.linuxdeploy import LinuxDeploy
from briefcase.integrations.subprocess import Subprocess


@pytest.fixture
def mock_tools(tmp_path, mock_tools) -> ToolCache:
    mock_tools.host_os = "Linux"
    mock_tools.host_arch = "i686"

    # Mock default tools
    mock_tools.subprocess = MagicMock(spec_set=Subprocess)
    mock_tools.file = MagicMock(spec_set=File)

    # Restore shutil
    mock_tools.shutil = shutil

    # Create a dummy bundle path
    (tmp_path / "bundle").mkdir()

    return mock_tools


@pytest.fixture
def linuxdeploy(mock_tools):
    return LinuxDeploy(mock_tools)



import pytest

from briefcase.integrations.windows_sdk import WindowsSDK


@pytest.mark.parametrize(
    "sdk_version, supported",
    [
        ("42.42.1000", True),
        ("42.42.1000.0", True),
        ("42.42.1001.001", True),
        ("42.42.1001.0.dev", True),
        ("42.42.1001.asdf", True),
        ("42.42.123456789", True),
        ("42.42.123456789.10", True),
        ("42.42.950", False),
        ("42.42.950.50", False),
        ("42.42.NaN.50", False),
        ("42", False),
        ("42.42", False),
        ("41.41.1000", False),
        (42.42, False),
        (42, False),
    ],
)
def test_winsdk_version_validation(
    mock_tools,
    sdk_version,
    supported,
    tmp_path,
    monkeypatch,
):
    """Evaluation of the proposed SDK version is correct."""
    # Patch target SDK version
    monkeypatch.setattr(WindowsSDK, "SDK_VERSION", "42.42")
    monkeypatch.setattr(WindowsSDK, "SDK_MIN_VERSION", 1000)

    windows_sdk = WindowsSDK(
        tools=mock_tools,
        root_path=tmp_path / "win_sdk",
        version=sdk_version,
        arch="x64",
    )

    # Verify the version
    assert WindowsSDK._is_supported_version(windows_sdk) is supported



import pytest

from briefcase.exceptions import UnsupportedHostError
from briefcase.integrations.subprocess import Subprocess


def test_short_circuit(mock_tools):
    """Tool is not created if already cached."""
    mock_tools.subprocess = "tool"

    tool = Subprocess.verify(mock_tools)

    assert tool == "tool"
    assert tool == mock_tools.subprocess


def test_unsupported_os(mock_tools):
    """When host OS is not supported, an error is raised."""
    mock_tools.host_os = "wonky"

    # Delete subprocess since it has already been verified
    delattr(mock_tools, "subprocess")

    with pytest.raises(
        UnsupportedHostError,
        match=f"{Subprocess.name} is not supported on wonky",
    ):
        Subprocess.verify(mock_tools)


def test_verify(mock_tools):
    """Subprocess verify always returns/sets subprocess tool."""
    subprocess_tool = Subprocess.verify(mock_tools)
    assert isinstance(subprocess_tool, Subprocess)
    assert subprocess_tool is mock_tools.subprocess



import subprocess
from signal import SIGTERM
from unittest import mock

import pytest

from briefcase.console import Console, Log, LogLevel
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.macOS import macOS_log_clean_filter
from briefcase.platforms.macOS.app import macOSAppRunCommand


@pytest.fixture
def run_command(tmp_path):
    command = macOSAppRunCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.home_path = tmp_path / "home"
    command.tools.subprocess = mock.MagicMock(spec_set=Subprocess)
    command._stream_app_logs = mock.MagicMock()

    # To satisfy coverage, the stop function must be invoked
    # at least once when streaming app logs.
    def mock_stream_app_logs(app, stop_func, **kwargs):
        stop_func()

    command._stream_app_logs.side_effect = mock_stream_app_logs
    command.tools.os.kill = mock.MagicMock()

    return command


def test_run_gui_app(run_command, first_app_config, sleep_zero, tmp_path, monkeypatch):
    """A macOS GUI app can be started."""
    # Mock a popen object that represents the log stream
    log_stream_process = mock.MagicMock(spec_set=subprocess.Popen)
    run_command.tools.subprocess.Popen.return_value = log_stream_process

    # Monkeypatch the tools get the process ID
    monkeypatch.setattr(
        "briefcase.platforms.macOS.get_process_id_by_command", lambda *a, **kw: 100
    )

    run_command.run_app(first_app_config, test_mode=False, passthrough=[])

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path],
        cwd=tmp_path / "home",
        check=True,
    )

    # The log stream was started
    run_command._stream_app_logs.assert_called_with(
        first_app_config,
        popen=log_stream_process,
        test_mode=False,
        clean_filter=macOS_log_clean_filter,
        clean_output=True,
        stop_func=mock.ANY,
        log_stream=True,
    )

    # The app process was killed on exit.
    run_command.tools.os.kill.assert_called_with(100, SIGTERM)


def test_run_gui_app_with_passthrough(
    run_command,
    first_app_config,
    sleep_zero,
    tmp_path,
    monkeypatch,
):
    """A macOS app can be started in debug mode with args."""
    run_command.logger.verbosity = LogLevel.DEBUG

    # Mock a popen object that represents the log stream
    log_stream_process = mock.MagicMock(spec_set=subprocess.Popen)
    run_command.tools.subprocess.Popen.return_value = log_stream_process

    # Monkeypatch the tools get the process ID
    monkeypatch.setattr(
        "briefcase.platforms.macOS.get_process_id_by_command", lambda *a, **kw: 100
    )

    # Run the app with args
    run_command.run_app(
        first_app_config,
        test_mode=False,
        passthrough=["foo", "--bar"],
    )

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path, "--args", "foo", "--bar"],
        cwd=tmp_path / "home",
        check=True,
        env={"BRIEFCASE_DEBUG": "1"},
    )

    # The log stream was started
    run_command._stream_app_logs.assert_called_with(
        first_app_config,
        popen=log_stream_process,
        test_mode=False,
        clean_filter=macOS_log_clean_filter,
        clean_output=True,
        stop_func=mock.ANY,
        log_stream=True,
    )

    # The app process was killed on exit.
    run_command.tools.os.kill.assert_called_with(100, SIGTERM)


def test_run_gui_app_failed(run_command, first_app_config, sleep_zero, tmp_path):
    """If there's a problem started the app, an exception is raised."""
    # Mock a failure opening the app
    run_command.tools.subprocess.run.side_effect = subprocess.CalledProcessError(
        cmd=["open", "-n", run_command.binary_path(first_app_config)],
        returncode=1,
    )

    with pytest.raises(BriefcaseCommandError):
        run_command.run_app(first_app_config, test_mode=False, passthrough=[])

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path],
        cwd=tmp_path / "home",
        check=True,
    )

    # No attempt was made to stream the log or cleanup
    run_command._stream_app_logs.assert_not_called()
    run_command.tools.os.kill.assert_not_called()


def test_run_gui_app_find_pid_failed(
    run_command,
    first_app_config,
    sleep_zero,
    tmp_path,
    monkeypatch,
    capsys,
):
    """If after app is started, its pid is not found, do not stream output."""
    # Mock a failed PID lookup
    monkeypatch.setattr(
        "briefcase.platforms.macOS.get_process_id_by_command",
        lambda *a, **kw: None,
    )

    with pytest.raises(BriefcaseCommandError) as exc_info:
        run_command.run_app(first_app_config, test_mode=False, passthrough=[])

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path],
        cwd=tmp_path / "home",
        check=True,
    )
    assert exc_info.value.msg == (
        "Unable to find process for app first-app to start log streaming."
    )

    # No attempt was made to stream the log or cleanup
    run_command._stream_app_logs.assert_not_called()
    run_command.tools.os.kill.assert_not_called()


@pytest.mark.parametrize("is_console_app", [True, False])
def test_run_app_test_mode(
    run_command,
    first_app_config,
    is_console_app,
    sleep_zero,
    tmp_path,
    monkeypatch,
):
    """A macOS app can be started in test mode."""
    # Test mode is the same regardless of whether it's test mode or not.
    first_app_config.console_app = is_console_app

    # Mock a popen object that represents the log stream
    log_stream_process = mock.MagicMock(spec_set=subprocess.Popen)
    run_command.tools.subprocess.Popen.return_value = log_stream_process

    # Monkeypatch the tools get the process ID
    monkeypatch.setattr(
        "briefcase.platforms.macOS.get_process_id_by_command", lambda *a, **kw: 100
    )

    run_command.run_app(first_app_config, test_mode=True, passthrough=[])

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path],
        cwd=tmp_path / "home",
        check=True,
        env={"BRIEFCASE_MAIN_MODULE": "tests.first_app"},
    )

    # The log stream was started
    run_command._stream_app_logs.assert_called_with(
        first_app_config,
        popen=log_stream_process,
        test_mode=True,
        clean_filter=macOS_log_clean_filter,
        clean_output=True,
        stop_func=mock.ANY,
        log_stream=True,
    )

    # The app process was killed on exit.
    run_command.tools.os.kill.assert_called_with(100, SIGTERM)


def test_run_console_app(run_command, first_app_config, tmp_path):
    """A macOS console app can be started."""
    # Set the app to be a console app
    first_app_config.console_app = True

    run_command.run_app(first_app_config, test_mode=False, passthrough=[])

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    run_command.tools.subprocess.run.assert_called_with(
        [bin_path / "Contents/MacOS/First App"],
        cwd=tmp_path / "home",
        check=True,
        stream_output=False,
    )

    # The log stream was not started
    run_command._stream_app_logs.assert_not_called()


def test_run_console_app_with_passthrough(
    run_command,
    first_app_config,
    tmp_path,
):
    """A macOS console app can be started in debug mode with args."""
    run_command.logger.verbosity = LogLevel.DEBUG

    # Set the app to be a console app
    first_app_config.console_app = True

    # Run the app with args
    run_command.run_app(
        first_app_config,
        test_mode=False,
        passthrough=["foo", "--bar"],
    )

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    run_command.tools.subprocess.run.assert_called_with(
        [bin_path / "Contents/MacOS/First App", "foo", "--bar"],
        cwd=tmp_path / "home",
        check=True,
        stream_output=False,
        env={"BRIEFCASE_DEBUG": "1"},
    )

    # The log stream was not started
    run_command._stream_app_logs.assert_not_called()


def test_run_console_app_failed(run_command, first_app_config, sleep_zero, tmp_path):
    """If there's a problem started a console app, an exception is raised."""
    # Set the app to be a console app
    first_app_config.console_app = True

    # Mock a failure opening the app
    run_command.tools.subprocess.run.side_effect = subprocess.CalledProcessError(
        cmd=[run_command.binary_path(first_app_config) / "Contents/MacOS/First App"],
        returncode=1,
    )

    # Although the command raises an error, this could be because the script itself
    # raised an error.
    run_command.run_app(first_app_config, test_mode=False, passthrough=[])

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    run_command.tools.subprocess.run.assert_called_with(
        [bin_path / "Contents/MacOS/First App"],
        cwd=tmp_path / "home",
        stream_output=False,
        check=True,
    )

    # No attempt was made to stream the log or cleanup
    run_command._stream_app_logs.assert_not_called()



import shutil
from unittest import mock

from ...utils import create_file


def test_no_support_path(create_command, myapp, no_support_path_index):
    """If support_path is not listed in briefcase.toml, no cleanup will be performed."""
    create_command.tools.file.download = mock.MagicMock()
    create_command.cleanup_app_support_package(myapp)
    create_command.tools.file.download.assert_not_called()


def test_support_path_does_not_exist(
    create_command,
    myapp,
    app_requirements_path_index,
):
    """If support_path is defined, but the folder doesn't exist, no cleanup is
    needed."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    create_command.cleanup_app_support_package(myapp)

    # No cleanup was performed.
    create_command.tools.shutil.assert_not_called()


def test_cleanup_support_package(
    create_command,
    myapp,
    support_path,
    app_requirements_path_index,
):
    """If a support package already exists, it can be cleaned up."""

    # Mock an existing support file
    create_file(support_path / "old/trash.txt", "Old support file")

    # Mock shutil so we can confirm that rmtree is called,
    # but we still want the side effect of calling it
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.rmtree.side_effect = shutil.rmtree

    # Re-install the support package
    create_command.cleanup_app_support_package(myapp)

    # The old support package was deleted
    create_command.tools.shutil.rmtree.assert_called_with(support_path)

    # Confirm that the old support files have been removed
    assert not support_path.exists()



from unittest.mock import MagicMock

import pytest

from ...utils import NoMatchString, PartialMatchString


def test_multiple_pep621_authors(convert_command, monkeypatch):
    """All authors are added to the options in addition to an option for Other."""
    mock_select_option = MagicMock()
    mock_select_option.return_value = "Firstname Firstauthor"
    monkeypatch.setattr(convert_command, "select_option", mock_select_option)

    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n"
        "authors = [\n"
        '    {name="Firstname Firstauthor", email="mail1@tld.com"},\n'
        '    {email="mail2@tld.com"},\n'
        '    {name="Name Thirdauthor", email="mail3@tld.com"},\n'
        '    {name="Firstname Fourthauthor", email="mail4@tld.com"},\n'
        "]",
        encoding="utf-8",
    )
    convert_command.input_author(None)

    mock_select_option.assert_called_once_with(
        intro=PartialMatchString(
            "We found these author names in the PEP621 formatted 'pyproject.toml'."
        ),
        variable="Author",
        options=[
            "Firstname Firstauthor",
            "Name Thirdauthor",
            "Firstname Fourthauthor",
            "Firstname Firstauthor, Name Thirdauthor & Firstname Fourthauthor",
            "Other",
        ],
        default=None,
        override_value=None,
    )


def test_single_pep621_author(convert_command, monkeypatch):
    """If there is only one author, then you don't get the joined authors-option."""
    mock_select_option = MagicMock()
    mock_select_option.return_value = "Firstname Firstauthor"
    monkeypatch.setattr(convert_command, "select_option", mock_select_option)

    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n"
        "authors = [\n"
        '    {name="Firstname Firstauthor", email="mail1@tld.com"},\n'
        "]",
        encoding="utf-8",
    )
    convert_command.input_author(None)

    mock_select_option.assert_called_once_with(
        intro=PartialMatchString(
            "We found these author names in the PEP621 formatted 'pyproject.toml'."
        ),
        variable="Author",
        options=[
            "Firstname Firstauthor",
            "Other",
        ],
        default=None,
        override_value=None,
    )


def test_multiple_pep621_authors_select_other(convert_command, monkeypatch):
    """If you select "Other", then you can type in a name."""
    mock_select_option = MagicMock()
    mock_select_option.return_value = "Other"
    monkeypatch.setattr(convert_command, "select_option", mock_select_option)
    mock_input_text = MagicMock()
    mock_input_text.return_value = "Some Name"
    monkeypatch.setattr(convert_command, "input_text", mock_input_text)

    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n"
        "authors = [\n"
        '    {name="Firstname Firstauthor", email="mail1@tld.com"},\n'
        '    {email="mail2@tld.com"},\n'
        '    {name="Name Thirdauthor", email="mail3@tld.com"},\n'
        '    {name="Firstname Fourthauthor", email="mail4@tld.com"},\n'
        "]",
        encoding="utf-8",
    )
    assert convert_command.input_author(None) == "Some Name"
    mock_select_option.assert_called_once_with(
        intro=PartialMatchString(
            "We found these author names in the PEP621 formatted 'pyproject.toml'."
        ),
        variable="Author",
        options=[
            "Firstname Firstauthor",
            "Name Thirdauthor",
            "Firstname Fourthauthor",
            "Firstname Firstauthor, Name Thirdauthor & Firstname Fourthauthor",
            "Other",
        ],
        default=None,
        override_value=None,
    )
    mock_input_text.assert_called_once_with(
        intro="Who do you want to be credited as the author of this application?",
        variable="author",
        default="Jane Developer",
        override_value=None,
    )


@pytest.mark.parametrize("write_empty_pyproject", [True, False])
def test_no_pep621_author(convert_command, monkeypatch, write_empty_pyproject):
    """If there is no author names in the pyproject.toml, then you're asked to write the
    name."""
    mock_input_text = MagicMock()
    monkeypatch.setattr(convert_command, "input_text", mock_input_text)

    if write_empty_pyproject:
        (convert_command.base_path / "pyproject.toml").write_text(
            "[project]\n" "authors = [\n" '    {email="mail1@tld.com"},\n' "]",
            encoding="utf-8",
        )
    convert_command.input_author(None)
    mock_input_text.assert_called_once_with(
        intro=NoMatchString(
            "We found these author names in the PEP621 formatted pyproject.toml."
        ),
        variable="author",
        default="Jane Developer",
        override_value=None,
    )


def test_override(convert_command):
    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n"
        "authors = [\n"
        '    {name="Firstname Firstauthor", email="mail1@tld.com"},\n'
        '    {email="mail2@tld.com"},\n'
        '    {name="Name Thirdauthor", email="mail3@tld.com"},\n'
        '    {name="Firstname Fourthauthor", email="mail4@tld.com"},\n'
        "]",
        encoding="utf-8",
    )
    assert convert_command.input_author("Some Developer") == "Some Developer"


def test_prompted_author_without_pyproject(convert_command):
    """The user is prompted for an author."""
    convert_command.input.values = ["Some author"]
    assert convert_command.input_author(None) == "Some author"


def test_prompted_author_with_pyproject(convert_command):
    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n"
        "authors = [\n"
        '    {name="Firstname Firstauthor", email="mail1@tld.com"},\n'
        '    {email="mail2@tld.com"},\n'
        '    {name="Name Thirdauthor", email="mail3@tld.com"},\n'
        '    {name="Firstname Fourthauthor", email="mail4@tld.com"},\n'
        "]",
        encoding="utf-8",
    )
    convert_command.input.values = ["2"]
    assert convert_command.input_author(None) == "Name Thirdauthor"


def test_prompted_author_with_pyproject_joined_author(convert_command):
    """You can select all authors joined together."""
    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n"
        "authors = [\n"
        '    {name="Firstname Firstauthor", email="mail1@tld.com"},\n'
        '    {email="mail2@tld.com"},\n'
        '    {name="Name Thirdauthor", email="mail3@tld.com"},\n'
        '    {name="Firstname Fourthauthor", email="mail4@tld.com"},\n'
        "]",
        encoding="utf-8",
    )
    convert_command.input.values = ["4"]
    assert (
        convert_command.input_author(None)
        == "Firstname Firstauthor, Name Thirdauthor & Firstname Fourthauthor"
    )


def test_prompted_author_with_pyproject_other(convert_command):
    """If you select "Other", then you can type in a name."""
    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n"
        "authors = [\n"
        '    {name="Firstname Firstauthor", email="mail1@tld.com"},\n'
        '    {email="mail2@tld.com"},\n'
        '    {name="Name Thirdauthor", email="mail3@tld.com"},\n'
        '    {name="Firstname Fourthauthor", email="mail4@tld.com"},\n'
        "]",
        encoding="utf-8",
    )
    convert_command.input.values = ["5", "Some Author"]
    assert convert_command.input_author(None) == "Some Author"



from unittest import mock

import pytest

import briefcase.integrations.xcode
from briefcase.console import Console, Log
from briefcase.platforms.macOS.xcode import macOSXcodePackageCommand

# skip most tests since packaging uses the same code as app command


@pytest.fixture
def package_command(tmp_path):
    command = macOSXcodePackageCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    return command


def test_verify(package_command, monkeypatch):
    """If you're on macOS, you can verify tools."""
    package_command.tools.host_os = "Darwin"

    mock_ensure_xcode_is_installed = mock.MagicMock()
    monkeypatch.setattr(
        briefcase.integrations.xcode.Xcode,
        "ensure_xcode_is_installed",
        mock_ensure_xcode_is_installed,
    )
    mock_ensure_command_line_tools_are_installed = mock.MagicMock()
    monkeypatch.setattr(
        briefcase.integrations.xcode.XcodeCliTools,
        "ensure_command_line_tools_are_installed",
        mock_ensure_command_line_tools_are_installed,
    )
    mock_confirm_xcode_license_accepted = mock.MagicMock()
    monkeypatch.setattr(
        briefcase.integrations.xcode.XcodeCliTools,
        "confirm_xcode_license_accepted",
        mock_confirm_xcode_license_accepted,
    )

    package_command.verify_tools()

    assert package_command.tools.xcode_cli is not None
    mock_ensure_xcode_is_installed.assert_called_once_with(
        tools=package_command.tools,
        min_version=(13, 0, 0),
    )
    mock_ensure_command_line_tools_are_installed.assert_called_once_with(
        tools=package_command.tools
    )
    mock_confirm_xcode_license_accepted.assert_called_once_with(
        tools=package_command.tools
    )



from briefcase.console import LogLevel


def test_output(mock_sub, streaming_process, sleep_zero, capsys):
    """Process output is printed."""
    streamer = mock_sub.stream_output_non_blocking("testing", streaming_process)
    streamer.join()

    # fmt: off
    assert capsys.readouterr().out == (
        "output line 1\n"
        "\n"
        "output line 3\n"
    )
    # fmt: on


def test_output_debug(mock_sub, streaming_process, sleep_zero, capsys):
    """Process output is printed; no debug output for only
    stream_output_non_blocking."""
    mock_sub.tools.logger.verbosity = LogLevel.DEBUG

    streamer = mock_sub.stream_output_non_blocking("testing", streaming_process)
    streamer.join()

    # fmt: off
    assert capsys.readouterr().out == (
        "output line 1\n"
        "\n"
        "output line 3\n"
    )
    # fmt: on


def test_captured_output(mock_sub, streaming_process, sleep_zero, capsys):
    """Process output is captured and available later."""
    streamer = mock_sub.stream_output_non_blocking(
        "testing",
        streaming_process,
        capture_output=True,
    )
    streamer.join()

    assert capsys.readouterr().out == ""
    # fmt: off
    assert streamer.captured_output == (
        "output line 1\n"
        "\n"
        "output line 3\n"
    )
    # fmt: on


def test_filter_func(mock_sub, streaming_process, sleep_zero, capsys):
    """Process output can be filtered with a filter func."""

    def filter_func(line):
        """Filter out lines without text."""
        if line == "":
            return
        yield line

    streamer = mock_sub.stream_output_non_blocking(
        "testing",
        streaming_process,
        filter_func=filter_func,
    )
    streamer.join()

    # fmt: off
    assert capsys.readouterr().out == (
        "output line 1\n"
        "output line 3\n"
    )
    # fmt: on



from unittest.mock import MagicMock

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseCommandError
from briefcase.platforms.linux.system import LinuxSystemBuildCommand


@pytest.mark.parametrize(
    "vendor, codename",
    [
        ("ubuntu", "jammy"),
        ("debian", "bullseye"),
    ],
)
def test_build_path(
    create_command,
    first_app_config,
    vendor,
    codename,
    tmp_path,
):
    """The bundle path contains vendor and Python source details."""
    first_app_config.target_vendor = vendor
    first_app_config.target_codename = codename

    assert (
        create_command.build_path(first_app_config)
        == tmp_path / "base_path/build/first-app" / vendor
    )


@pytest.mark.parametrize(
    "vendor, codename",
    [
        ("ubuntu", "jammy"),
        ("debian", "bullseye"),
    ],
)
def test_bundle_path(
    create_command,
    first_app_config,
    vendor,
    codename,
    tmp_path,
):
    """The bundle path contains vendor and Python source details."""
    first_app_config.target_vendor = vendor
    first_app_config.target_codename = codename

    assert (
        create_command.bundle_path(first_app_config)
        == tmp_path / "base_path/build/first-app" / vendor / codename
    )


def test_binary_path(create_command, first_app_config, tmp_path):
    """The binary path contains vendor and Python source details."""
    # Force the architecture to x86_64 for test purposes.
    create_command.tools.host_arch = "x86_64"

    # Force a dummy vendor:codename for test purposes.
    first_app_config.target_vendor = "somevendor"
    first_app_config.target_codename = "surprising"

    assert (
        create_command.binary_path(first_app_config)
        == tmp_path
        / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app"
    )


@pytest.mark.parametrize(
    "packaging_format, vendor_base, filename",
    [
        ("deb", "debian", "first-app_0.0.1-1~somevendor-surprising_wonky.deb"),
        ("rpm", "fedora", "first-app-0.0.1-1.elsurprising.wonky.rpm"),
        ("rpm", "suse", "first-app-0.0.1-1.wonky.rpm"),
        ("pkg", "arch", "first-app-0.0.1-1-wonky.pkg.tar.zst"),
    ],
)
def test_distribution_path(
    create_command,
    first_app_config,
    packaging_format,
    vendor_base,
    filename,
    tmp_path,
):
    """The distribution path contains vendor details."""
    # Mock return value for ABI from packaging system
    create_command._build_env_abi = MagicMock(return_value="wonky")

    # Set vendor base (RPM package naming changes for openSUSE)
    first_app_config.target_vendor_base = vendor_base

    # Force a dummy vendor:codename for test purposes.
    first_app_config.target_vendor = "somevendor"
    first_app_config.target_codename = "surprising"
    first_app_config.packaging_format = packaging_format

    assert (
        create_command.distribution_path(first_app_config)
        == tmp_path / "base_path/dist" / filename
    )


def test_distribution_path_unknown(create_command, first_app_config, tmp_path):
    """If the packaging format isn't known, an error is raised."""
    # Force the architecture to x86_64 for test purposes.
    create_command.tools.host_arch = "x86_64"

    # Force a dummy vendor:codename for test purposes.
    first_app_config.target_vendor = "somevendor"
    first_app_config.target_codename = "surprising"
    first_app_config.packaging_format = "unknown"

    with pytest.raises(
        BriefcaseCommandError,
        match=r"Briefcase doesn't currently know how to build system packages in UNKNOWN format.",
    ):
        create_command.distribution_path(first_app_config)


@pytest.mark.parametrize(
    "vendor, codename",
    [
        ("ubuntu", "jammy"),
        ("debian", "bullseye"),
    ],
)
def test_docker_image_tag(
    create_command,
    first_app_config,
    vendor,
    codename,
    tmp_path,
):
    """The docker image tag contains vendor and Python source details."""
    first_app_config.target_vendor = vendor
    first_app_config.target_codename = codename

    assert (
        create_command.docker_image_tag(first_app_config)
        == f"briefcase/com.example.first-app:{vendor}-{codename}"
    )


def test_docker_image_tag_uppercase_name(
    create_command,
    uppercase_app_config,
    tmp_path,
):
    uppercase_app_config.target_vendor = "somevendor"
    uppercase_app_config.target_codename = "surprising"

    assert (
        create_command.docker_image_tag(uppercase_app_config)
        == "briefcase/com.example.first-app:somevendor-surprising"
    )


def test_clone_options(create_command, tmp_path):
    """Docker options are cloned."""
    build_command = LinuxSystemBuildCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    build_command.target_image = "somevendor:surprising"
    build_command.extra_docker_build_args = ["--option-one", "--option-two"]

    create_command = build_command.create_command

    # Confirm the use_docker option has been cloned.
    assert create_command.is_clone
    assert create_command.target_image == "somevendor:surprising"
    assert create_command.extra_docker_build_args == ["--option-one", "--option-two"]



def test_pep621_data_is_used(convert_command):
    """The PEP621-description is used if present."""
    (convert_command.base_path / "pyproject.toml").write_text(
        '[project]\ndescription="Some description"', encoding="utf-8"
    )
    assert convert_command.input_description(None) == "Some description"


def test_override_is_used(convert_command):
    """The override is used even if a PEP621-description is present."""
    (convert_command.base_path / "pyproject.toml").write_text(
        '[project]\ndescription="Some description"', encoding="utf-8"
    )
    assert convert_command.input_description("OVERRIDE TEXT") == "OVERRIDE TEXT"


def test_prompted_description(convert_command):
    """The user is prompted for a description if there is no description in the
    pyproject.toml file."""
    convert_command.input.values = ["A very descriptive description"]
    assert convert_command.input_description(None) == "A very descriptive description"



from . import (
    android_sdk,
    cookiecutter,
    docker,
    file,
    flatpak,
    git,
    java,
    linuxdeploy,
    rcedit,
    subprocess,
    visualstudio,
    windows_sdk,
    wix,
    xcode,
)

__all__ = [
    "android_sdk",
    "cookiecutter",
    "docker",
    "file",
    "flatpak",
    "git",
    "java",
    "linuxdeploy",
    "rcedit",
    "subprocess",
    "visualstudio",
    "windows_sdk",
    "wix",
    "xcode",
]



import pytest

from briefcase.exceptions import MissingToolError, NetworkFailure, UnsupportedHostError
from briefcase.integrations.rcedit import RCEdit


def test_short_circuit(mock_tools):
    """Tool is not created if already cached."""
    mock_tools.rcedit = "tool"

    tool = RCEdit.verify(mock_tools)

    assert tool == "tool"
    assert tool == mock_tools.rcedit


@pytest.mark.parametrize("host_os", ["Darwin", "Linux", "wonky"])
def test_unsupported_os(mock_tools, host_os):
    """When host OS is not supported, an error is raised."""
    mock_tools.host_os = host_os

    with pytest.raises(
        UnsupportedHostError,
        match=f"{RCEdit.name} is not supported on {host_os}",
    ):
        RCEdit.verify(mock_tools)


def test_verify_exists(mock_tools, tmp_path):
    """If RCEdit already exists, verification doesn't download."""
    rcedit_path = tmp_path / "tools/rcedit-x64.exe"

    # Mock the existence of an install
    rcedit_path.touch()

    # Create a rcedit wrapper by verification
    rcedit = RCEdit.verify(mock_tools)

    # No download occurred
    assert mock_tools.file.download.call_count == 0
    assert mock_tools.os.chmod.call_count == 0

    # The build command retains the path to the downloaded file.
    assert rcedit.rcedit_path == rcedit_path


def test_verify_does_not_exist_dont_install(mock_tools, tmp_path):
    """If RCEdit doesn't exist, and install=False, it is *not* downloaded."""
    # Mock a successful download
    mock_tools.file.download.return_value = "new-downloaded-file"

    # True to create a rcedit wrapper by verification.
    # This will fail because it doesn't exist, but installation was disabled.
    with pytest.raises(MissingToolError):
        RCEdit.verify(mock_tools, install=False)

    # No download occurred
    assert mock_tools.file.download.call_count == 0
    assert mock_tools.os.chmod.call_count == 0


def test_verify_does_not_exist(mock_tools, tmp_path):
    """If RCEdit doesn't exist, it is downloaded."""
    rcedit_path = tmp_path / "tools/rcedit-x64.exe"

    # Mock a successful download
    def side_effect_create_mock_appimage(*args, **kwargs):
        rcedit_path.touch()
        return "new-downloaded-file"

    mock_tools.file.download.side_effect = side_effect_create_mock_appimage

    # Create a rcedit wrapper by verification
    rcedit = RCEdit.verify(mock_tools)

    # A download is invoked
    mock_tools.file.download.assert_called_with(
        url="https://github.com/electron/rcedit/"
        "releases/download/v2.0.0/rcedit-x64.exe",
        download_path=tmp_path / "tools",
        role="RCEdit",
    )

    # The build command retains the path to the downloaded file.
    assert rcedit.rcedit_path == rcedit_path


def test_verify_rcedit_download_failure(mock_tools, tmp_path):
    """If RCEdit doesn't exist, but a download failure occurs, an error is raised."""
    mock_tools.file.download.side_effect = NetworkFailure("mock")

    with pytest.raises(NetworkFailure, match="Unable to mock"):
        RCEdit.verify(mock_tools)

    # A download was invoked
    mock_tools.file.download.assert_called_with(
        url="https://github.com/electron/rcedit/"
        "releases/download/v2.0.0/rcedit-x64.exe",
        download_path=tmp_path / "tools",
        role="RCEdit",
    )



from unittest.mock import MagicMock

import pytest

from briefcase.platforms.linux import LinuxMixin


@pytest.fixture
def linux_mixin():
    """A Linux mixin with a mocked tools collection."""
    linux_mixin = LinuxMixin()
    linux_mixin.tools = MagicMock()
    return linux_mixin



from contextlib import nullcontext
from unittest.mock import MagicMock

import pytest


@pytest.mark.usefixtures("mock_docker")
@pytest.mark.usefixtures("mock_docker_app_context")
def test_run_app_context(mock_tools, my_app):
    """Run app context calls X11 passthrough and returns the keywords."""
    # Mock X11 passthrough manager with the null manager
    mock_tools.docker.x11_passthrough = MagicMock(wraps=nullcontext)

    in_kwargs = {"keyword_one": "val1", "keyword_two": "val2"}

    with mock_tools[my_app].app_context.run_app_context(in_kwargs) as kwargs:
        assert kwargs == in_kwargs

    mock_tools.docker.x11_passthrough.assert_called_once_with(in_kwargs)



import pytest

from briefcase.integrations.linuxdeploy import LinuxDeployQtPlugin

from ...utils import assert_url_resolvable


@pytest.fixture
def linuxdeploy_qt_plugin(mock_tools):
    return LinuxDeployQtPlugin(mock_tools)


def test_file_path(mock_tools, linuxdeploy_qt_plugin):
    """Default Linuxdeploy plugins reside in the linuxdeploy plugins path."""
    assert (
        linuxdeploy_qt_plugin.file_path
        == mock_tools.base_path / "linuxdeploy_plugins/qt"
    )


@pytest.mark.parametrize(
    "host_os, host_arch, linuxdeploy_arch",
    [
        ("Linux", "x86_64", "x86_64"),
        ("Linux", "i686", "i386"),
        ("Darwin", "x86_64", "x86_64"),
    ],
)
def test_file_name(mock_tools, host_os, host_arch, linuxdeploy_arch):
    """Linuxdeploy Qt plugin filename is architecture dependent."""
    mock_tools.host_os = host_os
    mock_tools.host_arch = host_arch

    linuxdeploy_qt_plugin = LinuxDeployQtPlugin(mock_tools)
    assert (
        linuxdeploy_qt_plugin.file_name
        == f"linuxdeploy-plugin-qt-{linuxdeploy_arch}.AppImage"
    )


def test_plugin_id(linuxdeploy_qt_plugin):
    """Linuxdeploy Qt plugin ID is fixed."""
    assert linuxdeploy_qt_plugin.plugin_id == "qt"


@pytest.mark.parametrize(
    "host_os, host_arch, linuxdeploy_arch",
    [
        ("Linux", "x86_64", "x86_64"),
        ("Linux", "i686", "i386"),
        ("Darwin", "x86_64", "x86_64"),
    ],
)
def test_download_url(mock_tools, host_os, host_arch, linuxdeploy_arch):
    """Linuxdeploy Qt plugin download URL is architecture dependent."""
    mock_tools.host_os = host_os
    mock_tools.host_arch = host_arch

    linuxdeploy_qt_plugin = LinuxDeployQtPlugin(mock_tools)

    assert linuxdeploy_qt_plugin.download_url == (
        "https://github.com/linuxdeploy/linuxdeploy-plugin-qt/"
        f"releases/download/continuous/linuxdeploy-plugin-qt-{linuxdeploy_arch}.AppImage"
    )
    assert_url_resolvable(linuxdeploy_qt_plugin.download_url)



import copy
import keyword
import re
import sys
import unicodedata
from types import SimpleNamespace

if sys.version_info >= (3, 11):  # pragma: no-cover-if-lt-py311
    import tomllib
else:  # pragma: no-cover-if-gte-py311
    import tomli as tomllib

from briefcase.platforms import get_output_formats, get_platforms

from .constants import RESERVED_WORDS
from .exceptions import BriefcaseConfigError

# PEP 508 restricts the naming of modules. The PEP defines a regex that uses
# re.IGNORECASE; but in in practice, packaging uses a version that rolls out the lower
# case, which has very slightly different semantics with non-ASCII characters. This
# definition is the one from
# https://github.com/pypa/packaging/blob/24.0/src/packaging/_tokenizer.py#L80
PEP508_NAME_RE = re.compile(r"^([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9._-]*[a-zA-Z0-9])$")


def is_valid_pep508_name(app_name):
    """Determine if the name is valid by PEP508 rules."""
    return PEP508_NAME_RE.match(app_name)


def is_reserved_keyword(app_name):
    """Determine if the name is a reserved keyword."""
    return keyword.iskeyword(app_name.lower()) or app_name.lower() in RESERVED_WORDS


def is_valid_app_name(app_name):
    return not is_reserved_keyword(app_name) and is_valid_pep508_name(app_name)


def make_class_name(formal_name):
    """Construct a valid class name from a formal name.

    :param formal_name: The formal name
    :returns: The app's class name
    """
    # Identifiers (including class names) can be unicode.
    # https://docs.python.org/3/reference/lexical_analysis.html#identifiers
    xid_start = {
        "Lu",  # uppercase letters
        "Ll",  # lowercase letters
        "Lt",  # titlecase letters
        "Lm",  # modifier letters
        "Lo",  # other letters
        "Nl",  # letter numbers
    }
    xid_continue = xid_start.union(
        {
            "Mn",  # nonspacing marks
            "Mc",  # spacing combining marks
            "Nd",  # decimal number
            "Pc",  # connector punctuations
        }
    )

    # Normalize to NFKC form, then remove any character that isn't
    # in the allowed categories, or is the underscore character;
    # Capitalize the resulting word.
    class_name = "".join(
        ch
        for ch in unicodedata.normalize("NFKC", formal_name)
        if unicodedata.category(ch) in xid_continue or ch in {"_"}
    )

    # If the first character isn't in the 'start' character set,
    # and it isn't already an underscore, prepend an underscore.
    if (
        class_name
        and unicodedata.category(class_name[0]) not in xid_start
        and class_name[0] != "_"
    ):
        class_name = f"_{class_name}"

    return class_name


VALID_BUNDLE_RE = re.compile(r"[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)+$")


def is_valid_bundle_identifier(bundle):
    # Ensure the bundle identifier follows the basi
    if not VALID_BUNDLE_RE.match(bundle):
        return False

    return True


# This is the canonical definition from PEP440, modified to include named groups
PEP440_CANONICAL_VERSION_PATTERN_RE = re.compile(
    r"^((?P<epoch>[1-9][0-9]*)!)?"
    r"(?P<release>(0|[1-9][0-9]*)(\.(0|[1-9][0-9]*))*)"
    r"((?P<pre_tag>a|b|rc)(?P<pre_value>0|[1-9][0-9]*))?"
    r"(\.post(?P<post>0|[1-9][0-9]*))?"
    r"(\.dev(?P<dev>0|[1-9][0-9]*))?$"
)


def is_pep440_canonical_version(version):
    """Determine if the string describes a valid PEP440 canonical version specifier.

    This implementation comes directly from PEP440 itself.

    :returns: True if the version string is valid; false otherwise.
    """
    return PEP440_CANONICAL_VERSION_PATTERN_RE.match(version) is not None


def parsed_version(version):
    """Return a parsed version string.

    :param version: The parsed version string
    """
    groupdict = PEP440_CANONICAL_VERSION_PATTERN_RE.match(version).groupdict()

    # Convert dot separated string of integers to tuple of integers
    groupdict["release"] = tuple(int(p) for p in groupdict.pop("release").split("."))

    # Convert strings to values
    for key in ("epoch", "pre_value", "post", "dev"):
        try:
            groupdict[key] = int(groupdict[key])
        except TypeError:
            pass

    tag = groupdict.pop("pre_tag")
    value = groupdict.pop("pre_value")
    groupdict["pre"] = (tag, value) if tag is not None else None
    return SimpleNamespace(**groupdict)


class BaseConfig:
    def __init__(self, **kwargs):
        for attr, value in kwargs.items():
            setattr(self, attr, value)


class GlobalConfig(BaseConfig):
    def __init__(
        self,
        project_name,
        version,
        bundle,
        license=None,
        url=None,
        author=None,
        author_email=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.project_name = project_name
        self.version = version
        self.bundle = bundle
        self.url = url
        self.author = author
        self.author_email = author_email
        self.license = license

        # Version number is PEP440 compliant:
        if not is_pep440_canonical_version(self.version):
            raise BriefcaseConfigError(
                f"Version number ({self.version}) is not valid.\n\n"
                "Version numbers must be PEP440 compliant; "
                "see https://www.python.org/dev/peps/pep-0440/ for details."
            )

    def __repr__(self):
        return f"<{self.project_name} v{self.version} GlobalConfig>"


class AppConfig(BaseConfig):
    def __init__(
        self,
        app_name,
        version,
        bundle,
        description,
        sources,
        license,
        formal_name=None,
        url=None,
        author=None,
        author_email=None,
        requires=None,
        icon=None,
        document_type=None,
        permission=None,
        template=None,
        template_branch=None,
        test_sources=None,
        test_requires=None,
        supported=True,
        long_description=None,
        console_app=False,
        **kwargs,
    ):
        super().__init__(**kwargs)

        # All app configs are created in unfinalized draft form.
        self.__draft__ = True

        self.app_name = app_name
        self.version = version
        self.bundle = bundle
        # Description can only be a single line. Ignore everything else.
        self.description = description.split("\n")[0]
        self.sources = sources
        self.formal_name = app_name if formal_name is None else formal_name
        self.url = url
        self.author = author
        self.author_email = author_email
        self.requires = requires
        self.icon = icon
        self.document_types = {} if document_type is None else document_type
        self.permission = {} if permission is None else permission
        self.template = template
        self.template_branch = template_branch
        self.test_sources = test_sources
        self.test_requires = test_requires
        self.supported = supported
        self.long_description = long_description
        self.license = license
        self.console_app = console_app

        if not is_valid_app_name(self.app_name):
            raise BriefcaseConfigError(
                f"{self.app_name!r} is not a valid app name.\n\n"
                "App names must not be reserved keywords such as 'and', 'for' and 'while'.\n"
                "They must also be PEP508 compliant (i.e., they can only include letters,\n"
                "numbers, '-' and '_'; must start with a letter; and cannot end with '-' or '_')."
            )

        if not is_valid_bundle_identifier(self.bundle):
            raise BriefcaseConfigError(
                f"{self.bundle!r} is not a valid bundle identifier.\n\n"
                "The bundle should be a reversed domain name. It must contain at least 2\n"
                "dot-separated sections; each section may only include letters, numbers,\n"
                "and hyphens; and each section may not contain any reserved words (like\n"
                "'switch', or 'while')."
            )

        # Version number is PEP440 compliant:
        if not is_pep440_canonical_version(self.version):
            raise BriefcaseConfigError(
                f"Version number for {self.app_name!r} ({self.version}) is not valid.\n\n"
                "Version numbers must be PEP440 compliant; "
                "see https://www.python.org/dev/peps/pep-0440/ for details."
            )

        # Sources list doesn't include any duplicates
        source_modules = {source.rsplit("/", 1)[-1] for source in self.sources}
        if len(self.sources) != len(source_modules):
            raise BriefcaseConfigError(
                f"The `sources` list for {self.app_name!r} contains duplicated "
                "package names."
            )

        # There is, at least, a source for the app module
        if self.module_name not in source_modules:
            raise BriefcaseConfigError(
                f"The `sources` list for {self.app_name!r} does not include a "
                f"package named {self.module_name!r}."
            )

    def __repr__(self):
        return f"<{self.bundle_identifier} v{self.version} AppConfig>"

    @property
    def module_name(self):
        """The module name for the app.

        This is derived from the name, but:
        * all `-` have been replaced with `_`.
        """
        return self.app_name.replace("-", "_")

    @property
    def bundle_name(self):
        """The bundle name for the app.

        This is derived from the app name, but:
        * all `_` have been replaced with `-`.
        """
        return self.app_name.replace("_", "-")

    @property
    def bundle_identifier(self):
        """The bundle identifier for the app.

        This is derived from the bundle and the bundle name, joined by a `.`.
        """
        return f"{self.bundle}.{self.bundle_name}"

    @property
    def class_name(self):
        """The class name for the app.

        This is derived from the formal name for the app.
        """
        return make_class_name(self.formal_name)

    @property
    def package_name(self):
        """The bundle name of the app, with `-` replaced with `_` to create something
        that can be used a namespace identifier on Python or Java, similar to
        `module_name`."""
        return self.bundle.replace("-", "_")

    def PYTHONPATH(self, test_mode):
        """The PYTHONPATH modifications needed to run this app.

        :param test_mode: Should test_mode sources be included?
        """
        paths = []
        sources = self.sources
        if test_mode and self.test_sources:
            sources.extend(self.test_sources)

        for source in sources:
            path = "/".join(source.rsplit("/", 1)[:-1])
            if path not in paths:
                paths.append(path)
        return paths

    def main_module(self, test_mode: bool):
        """The path to the main module for the app.

        In normal operation, this is ``app.module_name``; however,
        in test mode, it is prefixed with ``tests.``.

        :param test_mode: Are we running in test mode?
        """
        if test_mode:
            return f"tests.{self.module_name}"
        else:
            return self.module_name


def merge_config(config, data):
    """Merge a new set of configuration requirements into a base configuration.

    :param config: the base configuration to update. This configuration is modified in-
        situ.
    :param data: The new configuration data to merge into the configuration.
    """
    # Properties that are cumulative lists
    for option in [
        "requires",
        "sources",
        "test_requires",
        "test_sources",
    ]:
        value = data.pop(option, [])

        if value:
            config.setdefault(option, []).extend(value)

    # Properties that are cumulative tables
    for option in ["permission"]:
        value = data.pop(option, {})

        if value:
            config.setdefault(option, {}).update(value)

    config.update(data)


def merge_pep621_config(global_config, pep621_config):
    """Merge a PEP621 configuration into a Briefcase configuration."""

    def maybe_update(field, *project_fields):
        # If there's an existing key in the Briefcase config, it takes priority.
        if field in global_config:
            return

        # Traverse the fields in the pep621 config; if the config exists, set it
        # in the Briefcase config.
        datum = pep621_config
        try:
            for key in project_fields:
                datum = datum[key]
        except KeyError:
            pass
        else:
            global_config[field] = datum

    # Keys that map directly
    maybe_update("description", "description")
    maybe_update("license", "license")
    maybe_update("url", "urls", "Homepage")
    maybe_update("version", "version")

    # Use the details of the first author as the Briefcase author.
    if "author" not in global_config:
        try:
            global_config["author"] = pep621_config["authors"][0]["name"]
        except (KeyError, IndexError):
            pass
    if "author_email" not in global_config:
        try:
            global_config["author_email"] = pep621_config["authors"][0]["email"]
        except (KeyError, IndexError):
            pass

    # Briefcase requires is cumulative over PEP621 dependencies
    try:
        pep621_dependencies = pep621_config["dependencies"]
        requires = global_config.get("requires", [])
        global_config["requires"] = pep621_dependencies + requires
    except KeyError:
        pass

    # Briefcase test_requires is cumulative over PEP621 optional test dependencies
    try:
        pep621_test_dependencies = pep621_config["optional-dependencies"]["test"]
        test_requires = global_config.get("test_requires", [])
        global_config["test_requires"] = pep621_test_dependencies + test_requires
    except KeyError:
        pass


def parse_config(config_file, platform, output_format, logger):
    """Parse the briefcase section of the pyproject.toml configuration file.

    This method only does basic structural parsing of the TOML, looking for,
    at a minimum, a ``[tool.briefcase.app.<appname>]`` section declaring the
    existence of a single app. It will also search for:

      * ``[tool.briefcase]`` - global briefcase settings
      * ``[tool.briefcase.app.<appname>]`` - settings specific to the app
      * ``[tool.briefcase.app.<appname>.<platform>]`` - settings specific to
        the platform
      * ``[tool.briefcase.app.<appname>.<platform>.<format>]`` - settings
        specific to the output format

    A configuration can define multiple apps; the final output is the merged
    content of the global, app, platform and output format settings
    for each app, with output format definitions taking precedence over
    platform, over app-level, over global. The final result is a single
    (mostly) flat dictionary for each app.

    :param config_file: A file-like object containing TOML to be parsed.
    :param platform: The platform being targeted
    :param output_format: The output format
    :returns: A dictionary of configuration data. The top level dictionary is
        keyed by the names of the apps that are declared; each value is
        itself the configuration data merged from global, app, platform and
        format definitions.
    """
    try:
        pyproject = tomllib.load(config_file)
    except tomllib.TOMLDecodeError as e:
        raise BriefcaseConfigError(f"Invalid pyproject.toml: {e}") from e

    try:
        global_config = pyproject["tool"]["briefcase"]
    except KeyError as e:
        raise BriefcaseConfigError("No tool.briefcase section in pyproject.toml") from e

    # Merge the PEP621 configuration (if it exists)
    try:
        merge_pep621_config(global_config, pyproject["project"])
    except KeyError:
        pass

    # For consistent results, sort the platforms and formats
    all_platforms = sorted(get_platforms().keys())
    all_formats = sorted(get_output_formats(platform).keys())

    try:
        all_apps = global_config.pop("app")
    except KeyError as e:
        raise BriefcaseConfigError("No Briefcase apps defined in pyproject.toml") from e

    for name, config in [("project", global_config)] + list(all_apps.items()):
        if isinstance(config.get("license"), str):
            section_name = "the Project" if name == "project" else f"{name!r}"
            logger.warning(
                f"""
*************************************************************************
** {f"WARNING: License Definition for {section_name} is Deprecated":67} **
*************************************************************************

    Briefcase now uses PEP 621 format for license definitions.

    Previously, the name of the license was assigned to the 'license'
    field in pyproject.toml. For PEP 621, the name of the license is
    assigned to 'license.text' or the name of the file containing the
    license is assigned to 'license.file'.

    The current configuration for {section_name} has a 'license' field
    that is specified as a string:

        license = "{config['license']}"

    To use the PEP 621 format (and to remove this warning), specify that
    the LICENSE file contains the license for {section_name}:

        license.file = "LICENSE"

*************************************************************************
"""
            )
            config["license"] = {"file": "LICENSE"}

    # Build the flat configuration for each app,
    # based on the requested platform and output format
    app_configs = {}
    for app_name, app_data in all_apps.items():
        # At this point, the base configuration will contain a section
        # for each configured platform. Iterate over all the known platforms,
        # and remove these platform configurations. Keep a copy of the platform
        # configuration if it matches the requested platform, and merge it
        # into the app's configuration
        platform_data = None
        for p in all_platforms:
            try:
                platform_block = app_data.pop(p)

                if p == platform:
                    # If the platform matches the requested format, preserve
                    # it for later use.
                    platform_data = platform_block
                    merge_config(platform_data, platform_data)

                    # The platform configuration will contain a section
                    # for each configured output format. Iterate over all
                    # the output format of the platform, and remove these
                    # output format configurations. Keep a copy of the output
                    # format configuration if it matches the requested output
                    # format, and merge it into the platform's configuration.
                    format_data = None
                    for f in all_formats:
                        try:
                            format_block = platform_data.pop(f)

                            if f == output_format:
                                # If the output format matches the requested
                                # one, preserve it.
                                format_data = format_block

                        except KeyError:
                            pass

                    # If we found a specific configuration for the requested
                    # output format, add it to the platform configuration,
                    # overwriting any platform-level settings with format-level
                    # values.
                    if format_data:
                        merge_config(platform_data, format_data)

            except KeyError:
                pass

        # Now construct the final configuration.
        # First, convert the requirement definition at the global level
        merge_config(global_config, global_config)

        # The app's config starts as a copy of the base briefcase configuration.
        config = copy.deepcopy(global_config)

        # The app name is both the key, and a property of the configuration
        config["app_name"] = app_name

        # Merge the app-specific requirements
        merge_config(config, app_data)

        # If there is platform-specific configuration, merge the requirements,
        # then overwrite the platform-specific values.
        # This will already include any format-specific configuration.
        if platform_data:
            merge_config(config, platform_data)

        # Construct a configuration object, and add it to the list
        # of configurations that are being handled.
        app_configs[app_name] = config

    return global_config, app_configs



from pathlib import Path

import pytest


@pytest.mark.parametrize(
    "cmdline, overrides",
    [
        ([], {}),
        (["-Q", "license=MIT"], {"license": "MIT"}),
        (
            ["-Q", "license=MIT", "-Q", "bootstrap=Toga"],
            {"license": "MIT", "bootstrap": "Toga"},
        ),
    ],
)
def test_convert_app(convert_command, cmdline, overrides, patch_tempdir):
    """An application can be set up for briefcase created."""
    (convert_command.base_path / "app_name").mkdir()
    (convert_command.base_path / "pyproject.toml").write_text("", encoding="utf-8")
    (convert_command.base_path / "app_name/__main__.py").write_text(
        "", encoding="utf-8"
    )

    # Configure no command line options
    options, _ = convert_command.parse_options(cmdline)

    # Run the run command
    convert_command(**options)

    # The right sequence of things will be done
    assert convert_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # Pyproject doesn't contain Briefcase config
        ("validate-pyproject-file",),
        # Run the first app
        (
            "new",
            {
                "template": None,
                "template_branch": None,
                "project_overrides": overrides,
                "tmp_path": Path(patch_tempdir.name),
            },
        ),
    ]



from briefcase.config import merge_pep621_config


def test_empty():
    "Merging a PEP621 config with no interesting keys causes no changes"
    briefcase_config = {"key": "value"}

    merge_pep621_config(briefcase_config, {"other": "thingy"})

    assert briefcase_config == {"key": "value"}


def test_base_keys():
    "If the PEP621 config provides keys, they are added"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {
            "description": "It's cool",
            "version": "1.2.3",
            "urls": {"Homepage": "https://example.com"},
            "license": {"text": "BSD License"},
        },
    )

    assert briefcase_config == {
        "key": "value",
        "description": "It's cool",
        "version": "1.2.3",
        "license": {"text": "BSD License"},
        "url": "https://example.com",
    }


def test_base_keys_override():
    "If the PEP621 config provides keys, they are added"
    briefcase_config = {
        "key": "value",
        "description": "Good code I promise",
        "version": "2.3.4",
        "url": "https://beeware.org",
        "license": "BSD License",
    }

    merge_pep621_config(
        briefcase_config,
        {
            "description": "It's cool",
            "version": "1.2.3",
            "urls": {"Homepage": "https://example.com"},
            "license": {"text": "GPL3"},
        },
    )

    assert briefcase_config == {
        "key": "value",
        "description": "Good code I promise",
        "version": "2.3.4",
        "license": "BSD License",
        "url": "https://beeware.org",
    }


def test_missing_subkeys():
    "If a subkey is missing, the value is ignored"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {
            "urls": {"Sponsorship": "https://example.com"},
        },
    )

    assert briefcase_config == {"key": "value"}


def test_specified_license_file():
    "The license file is included in the briefcase config if specified in the PEP621 config"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {
            "license": {"file": "license.txt"},
        },
    )

    assert briefcase_config == {"key": "value", "license": {"file": "license.txt"}}


def test_empty_authors():
    "If the author list is empty, no author is recorded"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {"authors": []},
    )

    assert briefcase_config == {
        "key": "value",
    }


def test_single_author():
    "If there is a single author, their details are recorded"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {
            "authors": [
                {
                    "name": "Jane Developer",
                    "email": "jane@example.com",
                }
            ]
        },
    )

    assert briefcase_config == {
        "key": "value",
        "author": "Jane Developer",
        "author_email": "jane@example.com",
    }


def test_multiple_authors():
    "If there are multiple authors, the first author is recorded"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {
            "authors": [
                {
                    "name": "Jane Developer",
                    "email": "jane@example.com",
                },
                {
                    "name": "John Assistant",
                    "email": "john@somewhere.com",
                },
            ]
        },
    )

    assert briefcase_config == {
        "key": "value",
        "author": "Jane Developer",
        "author_email": "jane@example.com",
    }


def test_mising_author_name():
    "If the author is missing name, only the author email is captured"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {
            "authors": [
                {
                    "email": "jane@example.com",
                }
            ]
        },
    )

    assert briefcase_config == {
        "key": "value",
        "author_email": "jane@example.com",
    }


def test_missing_author_email():
    "If the author email is missing, the name is still recorded"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {
            "authors": [
                {
                    "name": "Jane Developer",
                }
            ]
        },
    )

    assert briefcase_config == {
        "key": "value",
        "author": "Jane Developer",
    }


def test_existing_author_name():
    "If the author is defined in the global config, the PEP621 value is ignored"
    briefcase_config = {"key": "value", "author": "Grace Hopper"}

    merge_pep621_config(
        briefcase_config,
        {
            "authors": [
                {
                    "name": "Jane Developer",
                    "email": "jane@example.com",
                }
            ]
        },
    )

    assert briefcase_config == {
        "key": "value",
        "author": "Grace Hopper",
        "author_email": "jane@example.com",
    }


def test_existing_author_email():
    "If the author email is missing, the name is still recorded"
    briefcase_config = {"key": "value", "author_email": "grace@hopper.org"}

    merge_pep621_config(
        briefcase_config,
        {
            "authors": [
                {
                    "name": "Jane Developer",
                    "email": "jane@example.com",
                }
            ]
        },
    )

    assert briefcase_config == {
        "key": "value",
        "author": "Jane Developer",
        "author_email": "grace@hopper.org",
    }


def test_no_dependencies():
    "If there are no global dependencies, requires are used as is."
    briefcase_config = {"key": "value", "requires": ["first", "second"]}

    merge_pep621_config(
        briefcase_config,
        {},
    )

    assert briefcase_config == {
        "key": "value",
        "requires": ["first", "second"],
    }


def test_dependencies_without_requires():
    "If the global config doesn't specify requirements, dependencies are used as is"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {"dependencies": ["dep1", "dep2"]},
    )

    assert briefcase_config == {
        "key": "value",
        "requires": ["dep1", "dep2"],
    }


def test_dependencies_with_requires():
    "If the global config specify requirements, requires augments dependencies"
    briefcase_config = {"key": "value", "requires": ["first", "second"]}

    merge_pep621_config(
        briefcase_config,
        {"dependencies": ["dep1", "dep2"]},
    )

    assert briefcase_config == {
        "key": "value",
        "requires": ["dep1", "dep2", "first", "second"],
    }


def test_no_test_dependencies():
    "If the global config doesn't specify test dependencies, test_requires are used as is"
    briefcase_config = {"key": "value", "test_requires": ["first", "second"]}

    merge_pep621_config(
        briefcase_config,
        {},
    )

    assert briefcase_config == {
        "key": "value",
        "test_requires": ["first", "second"],
    }


def test_optional_non_test_dependencies():
    "If the global config has non-test optional requirements, requires is used as-is"
    briefcase_config = {"key": "value", "requires": ["first", "second"]}

    merge_pep621_config(
        briefcase_config,
        {"optional-dependencies": {"other": ["dep1", "dep2"]}},
    )

    assert briefcase_config == {
        "key": "value",
        "requires": ["first", "second"],
    }


def test_test_dependencies_without_requires():
    "If the global config doesn't specify test requirements, test dependencies are used as is"
    briefcase_config = {"key": "value"}

    merge_pep621_config(
        briefcase_config,
        {"optional-dependencies": {"test": ["dep1", "dep2"]}},
    )

    assert briefcase_config == {
        "key": "value",
        "test_requires": ["dep1", "dep2"],
    }


def test_test_dependencies_with_requires():
    "If the global config specify requirements, requires augments dependencies"
    briefcase_config = {"key": "value", "test_requires": ["first", "second"]}

    merge_pep621_config(
        briefcase_config,
        {"optional-dependencies": {"test": ["dep1", "dep2"]}},
    )

    assert briefcase_config == {
        "key": "value",
        "test_requires": ["dep1", "dep2", "first", "second"],
    }



from unittest.mock import PropertyMock


def test_supported_encoding(base_command, capsys, monkeypatch):
    """User is not warned for a supported encoding."""
    monkeypatch.setattr(
        type(base_command.tools), "system_encoding", PropertyMock(return_value="UTF-8")
    )
    base_command.validate_locale()
    assert "Default system encoding is not supported" not in capsys.readouterr().out


def test_unsupported_encoding(base_command, capsys, monkeypatch):
    """User is warned for an unsupported encoding on Linux."""
    base_command.tools.host_os = "Linux"
    monkeypatch.setattr(
        type(base_command.tools),
        "system_encoding",
        PropertyMock(return_value="ISO-NOPE"),
    )
    base_command.validate_locale()
    assert "Default system encoding is not supported" in capsys.readouterr().out



from unittest.mock import MagicMock

import pytest

from briefcase.integrations.cookiecutter import PListExtension


@pytest.mark.parametrize(
    "value, expected",
    [
        (True, "<true/>"),
        (False, "<false/>"),
        ("Hello world", "<string>Hello world</string>"),
        (
            ["hello", "world", True],
            "<array>\n"
            "        <string>hello</string>\n"
            "        <string>world</string>\n"
            "        <true/>\n"
            "    </array>",
        ),
        (
            {"hello": "world", "goodbye": False},
            "<dict>\n"
            "        <key>hello</key>\n"
            "        <string>world</string>\n"
            "        <key>goodbye</key>\n"
            "        <false/>\n"
            "    </dict>",
        ),
    ],
)
def test_plist_value(value, expected):
    env = MagicMock()
    env.filters = {}
    PListExtension(env)
    assert env.filters["plist_value"](value) == expected



import os
import subprocess
import sys
from pathlib import Path
from unittest import mock

import pytest

from briefcase.console import Console, Log, LogLevel
from briefcase.exceptions import UnsupportedHostError
from briefcase.integrations.docker import Docker
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.linux import parse_freedesktop_os_release, system
from briefcase.platforms.linux.system import LinuxSystemRunCommand

from ....utils import create_file


@pytest.fixture
def run_command(tmp_path, first_app, monkeypatch):
    command = LinuxSystemRunCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
        apps={"app": first_app},
    )
    command.tools.home_path = tmp_path / "home"

    # Default to running on Linux
    command.tools.host_os = "Linux"

    # Set the host architecture for test purposes.
    command.tools.host_arch = "wonky"

    # Provide Docker
    monkeypatch.setattr(
        Docker, "_is_user_mapping_enabled", mock.MagicMock(return_value=True)
    )
    command.tools.docker = Docker(tools=command.tools)

    # Mock x11 passthrough
    # Mock DISPLAY environment variable
    monkeypatch.setenv("DISPLAY", "66.0")
    # Mock Subprocess.cleanup()
    command.tools.subprocess.cleanup = mock.MagicMock()
    # Mock the proxy
    mock_proxy_popen = mock.MagicMock(spec_set=subprocess.Popen)
    command.tools.docker._x11_tcp_proxy = mock.MagicMock(
        return_value=(mock_proxy_popen, 66)
    )
    # Mock xauth database file path
    mock_xauth_file_path = Path("/tmp/subdir/xauth_file.db")
    command.tools.docker._x11_proxy_display_xauth_file_path = mock.MagicMock(
        return_value=mock_xauth_file_path
    )
    # Mock the xauth database file write
    command.tools.docker._x11_write_xauth_file = mock.MagicMock()

    # Disable Docker by default
    command.target_image = None
    command.extra_docker_build_args = []

    command.tools.subprocess._subprocess = mock.MagicMock(spec_set=subprocess)
    command.tools.subprocess.run = mock.MagicMock(spec_set=Subprocess.run)
    command.tools.subprocess.check_output = mock.MagicMock(
        spec_set=Subprocess.check_output
    )

    command._stream_app_logs = mock.MagicMock()

    mock_linux_env(command, tmp_path, monkeypatch)

    return command


def mock_linux_env(run_command, tmp_path, monkeypatch):
    """Mock a linux system environment."""
    # Mock the freedesktop ID environment
    os_release = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
            "ID_LIKE=debian",
        ]
    )
    if sys.version_info >= (3, 10):
        # mock platform.freedesktop_os_release()
        run_command.tools.platform.freedesktop_os_release = mock.MagicMock(
            return_value=parse_freedesktop_os_release(os_release)
        )
    else:
        # For Pre Python3.10, mock the /etc/release file
        create_file(tmp_path / "os-release", os_release)
        run_command.tools.ETC_OS_RELEASE = tmp_path / "os-release"

    # Mock the glibc version
    run_command.target_glibc_version = mock.MagicMock(return_value="2.42")

    # Mock the existence of a valid non-docker system Python
    # with the same major/minor as the current Python
    python3 = mock.MagicMock()
    python3.resolve.return_value = Path(
        f"/usr/bin/python{sys.version_info.major}.{sys.version_info.minor}"
    )
    monkeypatch.setattr(system, "Path", mock.MagicMock(return_value=python3))


@pytest.mark.parametrize("host_os", ["Darwin", "Windows", "WeirdOS"])
def test_unsupported_host_os(run_command, first_app, host_os):
    """Error raised for an unsupported OS."""
    run_command.tools.host_os = host_os

    # Parse the command line
    run_command.parse_options([])

    with pytest.raises(
        UnsupportedHostError,
        match="Linux system projects can only be executed on Linux.",
    ):
        run_command()


def test_supported_host_os(run_command, first_app, sub_kw, tmp_path):
    """A supported OS (linux) can invoke run."""
    # This also verifies that Run will call the Build command

    # Set up call to start the app to return a known app process
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen = mock.MagicMock(
        return_value=log_popen
    )

    # Parse the command line
    run_command.parse_options([])

    # The command runs without error
    run_command()

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            f"{tmp_path / 'base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app'}"
        ],
        cwd=f"{tmp_path / 'home'}",
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        **sub_kw,
    )

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=False,
        clean_output=False,
    )


@pytest.mark.skipif(sys.platform == "win32", reason="Windows paths can't be dockerized")
def test_supported_host_os_docker(
    run_command,
    first_app,
    sub_kw,
    tmp_path,
    monkeypatch,
):
    """A supported OS (linux) can invoke run in Docker."""
    # This also verifies that Run will call Create and Build commands

    # Trigger to run in Docker
    run_command.target_image = first_app.target_image = "best/distro"

    # Python inside Docker is always newer
    run_command.tools.subprocess.check_output.return_value = "3.99"
    # Provide Docker app context
    run_command.verify_app_tools(app=first_app)

    # Set up call to start the app to return a known app process
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen = mock.MagicMock(
        return_value=log_popen
    )

    # Mock out the environment
    monkeypatch.setattr(
        run_command.tools.os, "environ", {"ENVVAR": "Value", "DISPLAY": ":99"}
    )

    # Parse the command line
    run_command.parse_options([])

    # The command runs without error
    run_command()

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            "docker",
            "run",
            "--rm",
            "--volume",
            "/tmp/subdir/xauth_file.db:/tmp/xauth_file.db:z",
            "--volume",
            f"{tmp_path / 'base_path/build/first-app/somevendor/surprising'}:/app:z",
            "--volume",
            f"{tmp_path / 'briefcase'}:/briefcase:z",
            "--env",
            "XAUTHORITY=/tmp/xauth_file.db",
            "--env",
            "DISPLAY=host.docker.internal:66",
            "--workdir",
            f"{tmp_path / 'home'}",
            "--add-host",
            "host.docker.internal:host-gateway",
            "briefcase/com.example.first-app:somevendor-surprising",
            "/app/first-app-0.0.1/usr/bin/first-app",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        env={"ENVVAR": "Value", "DISPLAY": ":99", "DOCKER_CLI_HINTS": "false"},
        **sub_kw,
    )

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=False,
        clean_output=False,
    )


def test_run_gui_app(run_command, first_app, sub_kw, tmp_path):
    """A bootstrap binary for a GUI app can be started."""

    # Set up tool cache
    run_command.verify_app_tools(app=first_app)

    # Set up the log streamer to return a known stream
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen = mock.MagicMock(
        return_value=log_popen
    )

    # Run the app
    run_command.run_app(first_app, test_mode=False, passthrough=[])

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            os.fsdecode(
                tmp_path
                / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app"
            )
        ],
        cwd=os.fsdecode(tmp_path / "home"),
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        **sub_kw,
    )

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=False,
        clean_output=False,
    )


def test_run_gui_app_passthrough(run_command, first_app, sub_kw, tmp_path):
    """A bootstrap binary for a GUI app can be started in debug mode with arguments."""
    run_command.logger.verbosity = LogLevel.DEBUG

    # Set up tool cache
    run_command.verify_app_tools(app=first_app)

    # Set up the log streamer to return a known stream
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen = mock.MagicMock(
        return_value=log_popen
    )

    # Run the app
    run_command.run_app(first_app, test_mode=False, passthrough=["foo", "--bar"])

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            os.fsdecode(
                tmp_path
                / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app"
            ),
            "foo",
            "--bar",
        ],
        cwd=os.fsdecode(tmp_path / "home"),
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        env=mock.ANY,
        **sub_kw,
    )
    # As we're adding to the environment, all the local values will be present.
    # Check that we've definitely set the values we care about
    env = run_command.tools.subprocess._subprocess.Popen.call_args.kwargs["env"]
    assert env["BRIEFCASE_DEBUG"] == "1"

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=False,
        clean_output=False,
    )


def test_run_gui_app_failed(run_command, first_app, sub_kw, tmp_path):
    """If there's a problem starting the GUI app, an exception is raised."""

    # Set up tool cache
    run_command.verify_app_tools(app=first_app)

    run_command.tools.subprocess._subprocess.Popen.side_effect = OSError

    with pytest.raises(OSError):
        run_command.run_app(first_app, test_mode=False, passthrough=[])

    # The run command was still invoked
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            os.fsdecode(
                tmp_path
                / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app"
            )
        ],
        cwd=os.fsdecode(tmp_path / "home"),
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        **sub_kw,
    )

    # No attempt to stream was made
    run_command._stream_app_logs.assert_not_called()


def test_run_console_app(run_command, first_app, tmp_path):
    """A bootstrap binary for a console app can be started."""
    first_app.console_app = True

    # Set up tool cache
    run_command.verify_app_tools(app=first_app)

    # Run the app
    run_command.run_app(first_app, test_mode=False, passthrough=[])

    # The process was started
    run_command.tools.subprocess.run.mock_calls == [
        mock.call(
            [
                tmp_path
                / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app"
            ],
            cwd=tmp_path / "home",
            bufsize=1,
            stream_output=False,
        )
    ]

    # No attempt to stream was made
    run_command._stream_app_logs.assert_not_called()


def test_run_console_app_passthrough(run_command, first_app, tmp_path):
    """A console app can be started in debug mode with command line arguments."""
    run_command.logger.verbosity = LogLevel.DEBUG

    first_app.console_app = True

    # Set up tool cache
    run_command.verify_app_tools(app=first_app)

    # Run the app
    run_command.run_app(first_app, test_mode=False, passthrough=["foo", "--bar"])

    # The process was started
    run_command.tools.subprocess.run.mock_calls == [
        mock.call(
            [
                tmp_path
                / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app",
                "foo",
                "--bar",
            ],
            cwd=tmp_path / "home",
            bufsize=1,
            stream_output=False,
            env={"BRIEFCASE_DEBUG": "1"},
        )
    ]

    # No attempt to stream was made
    run_command._stream_app_logs.assert_not_called()


def test_run_console_app_failed(run_command, first_app, sub_kw, tmp_path):
    """If there's a problem starting the console app, an exception is raised."""
    first_app.console_app = True

    # Set up tool cache
    run_command.verify_app_tools(app=first_app)

    run_command.tools.subprocess.run.side_effect = OSError

    with pytest.raises(OSError):
        run_command.run_app(first_app, test_mode=False, passthrough=[])

    # The run command was still invoked
    run_command.tools.subprocess.run.mock_calls == [
        mock.call(
            [
                tmp_path
                / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app"
            ],
            cwd=tmp_path / "home",
            bufsize=1,
            stream_output=False,
        )
    ]

    # No attempt to stream was made
    run_command._stream_app_logs.assert_not_called()


@pytest.mark.skipif(sys.platform == "win32", reason="Windows paths can't be dockerized")
def test_run_app_docker(run_command, first_app, sub_kw, tmp_path, monkeypatch):
    """A bootstrap binary can be started in Docker."""
    # Trigger to run in Docker
    run_command.target_image = first_app.target_image = "best/distro"

    # Python inside Docker is always newer
    run_command.tools.subprocess.check_output.return_value = "3.99"
    # Provide Docker app context
    run_command.verify_app_tools(app=first_app)

    # Set up the log streamer to return a known stream
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen = mock.MagicMock(
        return_value=log_popen
    )

    # Mock out the environment
    monkeypatch.setattr(
        run_command.tools.os, "environ", {"ENVVAR": "Value", "DISPLAY": ":99"}
    )

    # Run the app
    run_command.run_app(first_app, test_mode=False, passthrough=[])

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            "docker",
            "run",
            "--rm",
            "--volume",
            "/tmp/subdir/xauth_file.db:/tmp/xauth_file.db:z",
            "--volume",
            f"{tmp_path / 'base_path/build/first-app/somevendor/surprising'}:/app:z",
            "--volume",
            f"{tmp_path / 'briefcase'}:/briefcase:z",
            "--env",
            "XAUTHORITY=/tmp/xauth_file.db",
            "--env",
            "DISPLAY=host.docker.internal:66",
            "--workdir",
            f"{tmp_path / 'home'}",
            "--add-host",
            "host.docker.internal:host-gateway",
            "briefcase/com.example.first-app:somevendor-surprising",
            "/app/first-app-0.0.1/usr/bin/first-app",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        env={"ENVVAR": "Value", "DISPLAY": ":99", "DOCKER_CLI_HINTS": "false"},
        **sub_kw,
    )

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=False,
        clean_output=False,
    )


@pytest.mark.skipif(sys.platform == "win32", reason="Windows paths can't be dockerized")
def test_run_app_failed_docker(run_command, first_app, sub_kw, tmp_path, monkeypatch):
    """If there's a problem starting the app in Docker, an exception is raised."""

    # Trigger to run in Docker
    run_command.target_image = first_app.target_image = "best/distro"

    # Python inside Docker is always newer
    run_command.tools.subprocess.check_output.return_value = "3.99"
    # Provide Docker app context
    run_command.verify_app_tools(app=first_app)

    # Mock out the environment
    monkeypatch.setattr(
        run_command.tools.os, "environ", {"ENVVAR": "Value", "DISPLAY": ":99"}
    )

    run_command.tools.subprocess._subprocess.Popen.side_effect = OSError

    with pytest.raises(OSError):
        run_command.run_app(first_app, test_mode=False, passthrough=[])

    # The run command was still invoked
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            "docker",
            "run",
            "--rm",
            "--volume",
            "/tmp/subdir/xauth_file.db:/tmp/xauth_file.db:z",
            "--volume",
            f"{tmp_path / 'base_path/build/first-app/somevendor/surprising'}:/app:z",
            "--volume",
            f"{tmp_path / 'briefcase'}:/briefcase:z",
            "--env",
            "XAUTHORITY=/tmp/xauth_file.db",
            "--env",
            "DISPLAY=host.docker.internal:66",
            "--workdir",
            f"{tmp_path / 'home'}",
            "--add-host",
            "host.docker.internal:host-gateway",
            "briefcase/com.example.first-app:somevendor-surprising",
            "/app/first-app-0.0.1/usr/bin/first-app",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        env={"ENVVAR": "Value", "DISPLAY": ":99", "DOCKER_CLI_HINTS": "false"},
        **sub_kw,
    )

    # No attempt to stream was made
    run_command._stream_app_logs.assert_not_called()


@pytest.mark.parametrize("is_console_app", [True, False])
def test_run_app_test_mode(
    run_command,
    first_app,
    is_console_app,
    sub_kw,
    tmp_path,
    monkeypatch,
):
    """A linux App can be started in test mode."""
    # Test mode apps are always streamed
    first_app.console_app = is_console_app

    # Set up tool cache
    run_command.verify_app_tools(app=first_app)

    # Set up the log streamer to return a known stream
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen.return_value = log_popen

    # Mock out the environment
    monkeypatch.setattr(run_command.tools.os, "environ", {"ENVVAR": "Value"})

    # Run the app
    run_command.run_app(first_app, test_mode=True, passthrough=[])

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            os.fsdecode(
                tmp_path
                / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app"
            )
        ],
        cwd=os.fsdecode(tmp_path / "home"),
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        env={"ENVVAR": "Value", "BRIEFCASE_MAIN_MODULE": "tests.first_app"},
        **sub_kw,
    )

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=True,
        clean_output=False,
    )


@pytest.mark.skipif(sys.platform == "win32", reason="Windows paths can't be dockerized")
@pytest.mark.parametrize("is_console_app", [True, False])
def test_run_app_test_mode_docker(
    run_command,
    first_app,
    is_console_app,
    sub_kw,
    tmp_path,
    monkeypatch,
):
    """A linux App can be started in Docker in test mode."""
    # Test mode apps are always streamed
    first_app.console_app = is_console_app

    # Trigger to run in Docker
    run_command.target_image = first_app.target_image = "best/distro"

    # Python inside Docker is always newer
    run_command.tools.subprocess.check_output.return_value = "3.99"
    # Provide Docker app context
    run_command.verify_app_tools(app=first_app)

    # Set up the log streamer to return a known stream
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen.return_value = log_popen

    # Mock out the environment
    monkeypatch.setattr(
        run_command.tools.os, "environ", {"ENVVAR": "Value", "DISPLAY": ":99"}
    )

    # Run the app
    run_command.run_app(first_app, test_mode=True, passthrough=[])

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            "docker",
            "run",
            "--rm",
            "--volume",
            "/tmp/subdir/xauth_file.db:/tmp/xauth_file.db:z",
            "--volume",
            f"{tmp_path / 'base_path/build/first-app/somevendor/surprising'}:/app:z",
            "--volume",
            f"{tmp_path / 'briefcase'}:/briefcase:z",
            "--env",
            "BRIEFCASE_MAIN_MODULE=tests.first_app",
            "--env",
            "XAUTHORITY=/tmp/xauth_file.db",
            "--env",
            "DISPLAY=host.docker.internal:66",
            "--workdir",
            f"{tmp_path / 'home'}",
            "--add-host",
            "host.docker.internal:host-gateway",
            "briefcase/com.example.first-app:somevendor-surprising",
            "/app/first-app-0.0.1/usr/bin/first-app",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        env={"ENVVAR": "Value", "DISPLAY": ":99", "DOCKER_CLI_HINTS": "false"},
        **sub_kw,
    )

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=True,
        clean_output=False,
    )


@pytest.mark.parametrize("is_console_app", [True, False])
def test_run_app_test_mode_with_args(
    run_command,
    first_app,
    is_console_app,
    sub_kw,
    tmp_path,
    monkeypatch,
):
    """A linux App can be started in test mode with args."""
    # Test mode apps are always streamed
    first_app.console_app = is_console_app

    # Set up tool cache
    run_command.verify_app_tools(app=first_app)

    # Set up the log streamer to return a known stream
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen.return_value = log_popen

    # Mock out the environment
    monkeypatch.setattr(run_command.tools.os, "environ", {"ENVVAR": "Value"})

    # Run the app with args
    run_command.run_app(
        first_app,
        test_mode=True,
        passthrough=["foo", "--bar"],
    )

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            os.fsdecode(
                tmp_path
                / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr/bin/first-app"
            ),
            "foo",
            "--bar",
        ],
        cwd=os.fsdecode(tmp_path / "home"),
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        env={"ENVVAR": "Value", "BRIEFCASE_MAIN_MODULE": "tests.first_app"},
        **sub_kw,
    )

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=True,
        clean_output=False,
    )


@pytest.mark.skipif(sys.platform == "win32", reason="Windows paths can't be dockerized")
@pytest.mark.parametrize("is_console_app", [True, False])
def test_run_app_test_mode_with_args_docker(
    run_command,
    first_app,
    is_console_app,
    sub_kw,
    tmp_path,
    monkeypatch,
):
    """A linux App can be started in Docker in test mode with args."""
    # Test mode apps are always streamed
    first_app.console_app = is_console_app

    # Trigger to run in Docker
    run_command.target_image = first_app.target_image = "best/distro"

    # Python inside Docker is always newer
    run_command.tools.subprocess.check_output.return_value = "3.99"
    # Provide Docker app context
    run_command.verify_app_tools(app=first_app)

    # Set up the log streamer to return a known stream
    log_popen = mock.MagicMock()
    run_command.tools.subprocess._subprocess.Popen.return_value = log_popen

    # Mock out the environment
    monkeypatch.setattr(
        run_command.tools.os, "environ", {"ENVVAR": "Value", "DISPLAY": ":99"}
    )

    # Run the app with args
    run_command.run_app(
        first_app,
        test_mode=True,
        passthrough=["foo", "--bar"],
    )

    # The process was started
    run_command.tools.subprocess._subprocess.Popen.assert_called_with(
        [
            "docker",
            "run",
            "--rm",
            "--volume",
            "/tmp/subdir/xauth_file.db:/tmp/xauth_file.db:z",
            "--volume",
            f"{tmp_path / 'base_path/build/first-app/somevendor/surprising'}:/app:z",
            "--volume",
            f"{tmp_path / 'briefcase'}:/briefcase:z",
            "--env",
            "BRIEFCASE_MAIN_MODULE=tests.first_app",
            "--env",
            "XAUTHORITY=/tmp/xauth_file.db",
            "--env",
            "DISPLAY=host.docker.internal:66",
            "--workdir",
            f"{tmp_path / 'home'}",
            "--add-host",
            "host.docker.internal:host-gateway",
            "briefcase/com.example.first-app:somevendor-surprising",
            "/app/first-app-0.0.1/usr/bin/first-app",
            "foo",
            "--bar",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
        env={"ENVVAR": "Value", "DISPLAY": ":99", "DOCKER_CLI_HINTS": "false"},
        **sub_kw,
    )

    # The streamer was started
    run_command._stream_app_logs.assert_called_once_with(
        first_app,
        popen=log_popen,
        test_mode=True,
        clean_output=False,
    )



from automation.bootstraps import BRIEFCASE_EXIT_SUCCESS_SIGNAL, EXIT_SUCCESS_NOTIFY
from briefcase.bootstraps import PygameGuiBootstrap


class PygameAutomationBootstrap(PygameGuiBootstrap):
    def app_source(self):
        return f"""\
import importlib.metadata
import os
import sys
from pathlib import Path

import pygame


SCREEN_WIDTH, SCREEN_HEIGHT = 800, 600
WHITE = (255, 255, 255)


def main():
    # Linux desktop environments use an app's .desktop file to integrate the app
    # in to their application menus. The .desktop file of this app will include
    # the StartupWMClass key, set to app's formal name. This helps associate the
    # app's windows to its menu item.
    #
    # For association to work, any windows of the app must have WMCLASS property
    # set to match the value set in app's desktop file. For PyGame, this is set
    # using the SDL_VIDEO_X11_WMCLASS environment variable.

    # Find the name of the module that was used to start the app
    app_module = sys.modules["__main__"].__package__
    # Retrieve the app's metadata
    metadata = importlib.metadata.metadata(app_module)

    os.environ["SDL_VIDEO_X11_WMCLASS"] = metadata["Formal-Name"]

    pygame.init()
    pygame.display.set_caption(metadata["Formal-Name"])
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))

    pygame.time.set_timer(pygame.QUIT, 2000)

    running = True
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                print("{EXIT_SUCCESS_NOTIFY}")
                print("{BRIEFCASE_EXIT_SUCCESS_SIGNAL}")
                running = False
                break

        screen.fill(WHITE)
        pygame.display.flip()

    pygame.quit()
"""



import sys
from pathlib import Path

import pytest

from briefcase.console import LogLevel

PYTHONPATH = "PYTHONPATH"
PYTHONMALLOC = "PYTHONMALLOC"


@pytest.mark.skipif(sys.platform != "win32", reason="Relevant only for windows")
def test_pythonpath_with_one_source_in_windows(dev_command, first_app):
    """Test get environment with one source."""
    env = dev_command.get_environment(first_app, test_mode=False)
    assert env[PYTHONPATH] == f"{Path.cwd() / 'src'}"
    assert env[PYTHONMALLOC] == "default"


@pytest.mark.skipif(sys.platform != "win32", reason="Relevant only for windows")
def test_pythonpath_with_one_source_test_mode_in_windows(dev_command, first_app):
    """Test get environment with one source, no tests sources, in test mode."""
    env = dev_command.get_environment(first_app, test_mode=True)
    assert env[PYTHONPATH] == f"{Path.cwd() / 'src'}"
    assert env[PYTHONMALLOC] == "default"


@pytest.mark.skipif(sys.platform != "win32", reason="Relevant only for windows")
def test_pythonpath_with_two_sources_in_windows(dev_command, third_app):
    """Test get environment with two sources in windows."""
    env = dev_command.get_environment(third_app, test_mode=False)
    assert env[PYTHONPATH] == f"{Path.cwd() / 'src'};{Path.cwd()}"
    assert env[PYTHONMALLOC] == "default"


@pytest.mark.skipif(sys.platform != "win32", reason="Relevant only for windows")
def test_pythonpath_with_two_sources_and_tests_in_windows(dev_command, third_app):
    """Test get environment with two sources and test sources in windows."""
    third_app.test_sources = ["tests", "path/to/other"]
    env = dev_command.get_environment(third_app, test_mode=True)
    assert (
        env[PYTHONPATH]
        == f"{Path.cwd() / 'src'};{Path.cwd()};{Path.cwd() / 'path' / 'to'}"
    )
    assert env[PYTHONMALLOC] == "default"


@pytest.mark.skipif(sys.platform == "win32", reason="Relevant only for non-windows")
def test_pythonpath_with_one_source(dev_command, first_app):
    """Test get environment with one source."""
    env = dev_command.get_environment(first_app, test_mode=False)
    assert env[PYTHONPATH] == f"{Path.cwd() / 'src'}"
    assert PYTHONMALLOC not in env


@pytest.mark.skipif(sys.platform == "win32", reason="Relevant only for non-windows")
def test_pythonpath_with_one_source_test_mode(dev_command, first_app):
    """Test get environment with one source, no tests sources, in test mode."""
    env = dev_command.get_environment(first_app, test_mode=True)
    assert env[PYTHONPATH] == f"{Path.cwd() / 'src'}"
    assert PYTHONMALLOC not in env


@pytest.mark.skipif(sys.platform == "win32", reason="Relevant only for non-windows")
def test_pythonpath_with_two_sources_in_linux(dev_command, third_app):
    """Test get environment with two sources in linux."""
    env = dev_command.get_environment(third_app, test_mode=False)
    assert env[PYTHONPATH] == f"{Path.cwd() / 'src'}:{Path.cwd()}"
    assert PYTHONMALLOC not in env


@pytest.mark.skipif(sys.platform == "win32", reason="Relevant only for non-windows")
def test_pythonpath_with_two_sources_and_tests_in_linux(dev_command, third_app):
    """Test get environment with two sources and test sources in linux."""
    env = dev_command.get_environment(third_app, test_mode=True)
    assert (
        env[PYTHONPATH]
        == f"{Path.cwd() / 'src'}:{Path.cwd()}:{Path.cwd() / 'path' / 'to'}"
    )
    assert PYTHONMALLOC not in env


def test_non_verbose_mode(dev_command, first_app):
    """Non-verbose mode doesn't include BRIEFCASE_DEBUG in the dev environment."""
    dev_command.logger.verbosity = LogLevel.INFO
    env = dev_command.get_environment(first_app, test_mode=False)
    assert "BRIEFCASE_DEBUG" not in env


def test_verbose_mode(dev_command, first_app):
    """Verbose mode adds BRIEFCASE_DEBUG to the dev environment."""
    dev_command.logger.verbosity = LogLevel.DEBUG
    env = dev_command.get_environment(first_app, test_mode=False)
    assert env["BRIEFCASE_DEBUG"] == "1"



import subprocess

import pytest

from briefcase.console import LogLevel
from briefcase.exceptions import BriefcaseCommandError


@pytest.mark.parametrize("tool_debug_mode", (True, False))
def test_verify_runtime(flatpak, tool_debug_mode):
    """A Flatpak runtime and SDK can be verified."""
    # Enable verbose tool logging
    if tool_debug_mode:
        flatpak.tools.logger.verbosity = LogLevel.DEEP_DEBUG

    flatpak.verify_runtime(
        repo_alias="test-alias",
        runtime="org.beeware.flatpak.Platform",
        runtime_version="37.42",
        sdk="org.beeware.flatpak.SDK",
    )

    # The expected call was made
    flatpak.tools.subprocess.run.assert_called_once_with(
        [
            "flatpak",
            "install",
            "--assumeyes",
            "--user",
            "test-alias",
            "org.beeware.flatpak.Platform/gothic/37.42",
            "org.beeware.flatpak.SDK/gothic/37.42",
        ]
        + (["--verbose"] if tool_debug_mode else []),
        check=True,
        stream_output=False,
    )


def test_verify_runtime_fail(flatpak):
    """If runtime verification fails, an error is raised."""
    flatpak.tools.subprocess.run.side_effect = subprocess.CalledProcessError(
        cmd="flatpak install", returncode=1
    )

    with pytest.raises(
        BriefcaseCommandError,
        match=(
            r"Unable to install Flatpak runtime org.beeware.flatpak.Platform/gothic/37.42 "
            r"and SDK org.beeware.flatpak.SDK/gothic/37.42 from repo test-alias."
        ),
    ):
        flatpak.verify_runtime(
            repo_alias="test-alias",
            runtime="org.beeware.flatpak.Platform",
            runtime_version="37.42",
            sdk="org.beeware.flatpak.SDK",
        )



import pytest

from briefcase.commands.create import Git
from briefcase.exceptions import BriefcaseCommandError


def test_no_git(new_command, monkeypatch):
    """If Git is not installed, an error is raised."""

    def monkeypatch_verify_git(*a, **kw):
        raise BriefcaseCommandError("Briefcase requires git, but it is not installed")

    monkeypatch.setattr(Git, "verify", monkeypatch_verify_git)

    # The command will fail tool verification.
    with pytest.raises(
        BriefcaseCommandError, match=r"Briefcase requires git, but it is not installed"
    ):
        new_command()


def test_parse_config(new_command):
    """Attempting to parse the config is a no-op when invoking new."""
    assert new_command.parse_config("some_file.toml", {}) is None


@pytest.mark.parametrize(
    "cmdline, overrides",
    [
        ([], {}),
        (["-Q", "license=MIT"], {"license": "MIT"}),
        (
            ["-Q", "license=MIT", "-Q", "bootstrap=Toga"],
            {"license": "MIT", "bootstrap": "Toga"},
        ),
    ],
)
def test_new_app(new_command, cmdline, overrides):
    """A new application can be created."""

    # Configure no command line options
    options, _ = new_command.parse_options(cmdline)

    # Run the run command
    new_command(**options)

    # The right sequence of things will be done
    assert new_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # Run the first app
        (
            "new",
            {"template": None, "template_branch": None, "project_overrides": overrides},
        ),
    ]



import shutil
import subprocess
import sys
from pathlib import Path
from unittest import mock

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseCommandError
from briefcase.platforms.linux import system
from briefcase.platforms.linux.system import (
    LinuxSystemPackageCommand,
    debian_multiline_description,
)

from ....utils import create_file


@pytest.fixture
def package_command(first_app, tmp_path):
    command = LinuxSystemPackageCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.home_path = tmp_path / "home"

    # Mock ABI from packaging system
    command._deb_abi = "wonky"

    # Mock the app context
    command.tools.app_tools[first_app].app_context = mock.MagicMock()

    # Mock shutil
    command.tools.shutil = mock.MagicMock()
    # Make the mock rmtree still remove content
    command.tools.shutil.rmtree.side_effect = shutil.rmtree

    # Mock not using docker
    command.target_image = None
    command.extra_docker_build_args = []

    return command


@pytest.fixture
def first_app_deb(first_app):
    # Mock a debian app
    first_app.python_version_tag = "3.10"
    first_app.target_vendor_base = "debian"
    first_app.packaging_format = "deb"
    first_app.glibc_version = "2.99"
    first_app.long_description = "Long description\nfor the app"

    return first_app


def test_verify_no_docker(package_command, first_app_deb, monkeypatch):
    """If not using docker, existence of dpkg-deb is verified."""
    # Mock the existence of dpkg-deb
    package_command.tools.shutil.which = mock.MagicMock(return_value="/mybin/dpkg-deb")

    # App tools can be verified
    package_command.verify_app_tools(first_app_deb)

    # dpkg_deb was inspected
    package_command.tools.shutil.which.assert_called_once_with("dpkg-deb")


@pytest.mark.parametrize(
    "vendor_base, error_msg",
    [
        (
            "debian",
            "Can't find the dpkg tools. Try running `sudo apt install dpkg-dev`.",
        ),
        (None, "Can't find the dpkg-deb tool. Install this first to package the deb."),
    ],
)
def test_verify_dpkg_deb_missing(
    package_command,
    first_app_deb,
    vendor_base,
    error_msg,
    monkeypatch,
):
    """If dpkg-deb isn't installed, an error is raised."""
    # Mock distro so packager is found or not appropriately
    first_app_deb.target_vendor_base = vendor_base

    # Mock packager as missing
    package_command.tools.shutil.which = mock.MagicMock(return_value="")

    # Verifying app tools will raise an error
    with pytest.raises(BriefcaseCommandError, match=error_msg):
        package_command.verify_app_tools(first_app_deb)

    # which was called for dpkg-deb
    package_command.tools.shutil.which.assert_called_once_with("dpkg-deb")


def test_verify_docker(package_command, first_app_deb, monkeypatch):
    """If using Docker, no tool checks are needed."""
    # Mock using docker
    package_command.target_image = "somevendor:surprising"

    # Mock the existence of a valid non-docker system Python
    # with the same major/minor as the current Python,
    # plus the existence of dpkg-deb
    python3 = mock.MagicMock()
    python3.resolve.return_value = Path(
        f"/usr/bin/python{sys.version_info.major}.{sys.version_info.minor}"
    )

    dpkg_deb = mock.MagicMock()
    dpkg_deb.exists.return_value = False

    mock_Path = mock.MagicMock(side_effect=[python3, dpkg_deb])
    monkeypatch.setattr(system, "Path", mock_Path)

    # App tools can be verified
    package_command.verify_app_tools(first_app_deb)

    # dpkg_deb was not inspected
    dpkg_deb.exists.assert_not_called()


@pytest.mark.skipif(sys.platform == "win32", reason="Can't build debs on Windows")
def test_deb_package(package_command, first_app_deb, tmp_path):
    """A deb app can be packaged."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Package the app
    package_command.package_app(first_app_deb)

    # The control file is written
    assert (bundle_path / "first-app-0.0.1/DEBIAN/control").exists()
    with (bundle_path / "first-app-0.0.1/DEBIAN/control").open(encoding="utf-8") as f:
        assert (
            f.read()
            == "\n".join(
                [
                    "Package: first-app",
                    "Version: 0.0.1",
                    "Architecture: wonky",
                    "Maintainer: Megacorp <maintainer@example.com>",
                    "Homepage: https://example.com/first-app",
                    "Description: The first simple app \\ demonstration",
                    " Long description",
                    " for the app",
                    "Depends: libc6 (>=2.99), libpython3.10",
                    "Section: utils",
                    "Priority: optional",
                ]
            )
            + "\n"
        )

    package_command.tools.app_tools[
        first_app_deb
    ].app_context.run.assert_called_once_with(
        [
            "dpkg-deb",
            "--build",
            "--root-owner-group",
            "first-app-0.0.1",
        ],
        check=True,
        cwd=bundle_path,
    )

    # The deb was moved into the final location
    package_command.tools.shutil.move.assert_called_once_with(
        bundle_path / "first-app-0.0.1.deb",
        tmp_path / "base_path/dist/first-app_0.0.1-1~somevendor-surprising_wonky.deb",
    )


def test_deb_re_package(package_command, first_app_deb, tmp_path):
    """A deb app that has previously been packaged can be re-packaged."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Create an old control file that will be overwritten.
    create_file(bundle_path / "first-app-0.0.1/DEBIAN/control", "Old control content")

    # Package the app
    package_command.package_app(first_app_deb)

    # The control file is re-written
    assert (bundle_path / "first-app-0.0.1/DEBIAN/control").exists()
    with (bundle_path / "first-app-0.0.1/DEBIAN/control").open(encoding="utf-8") as f:
        assert (
            f.read()
            == "\n".join(
                [
                    "Package: first-app",
                    "Version: 0.0.1",
                    "Architecture: wonky",
                    "Maintainer: Megacorp <maintainer@example.com>",
                    "Homepage: https://example.com/first-app",
                    "Description: The first simple app \\ demonstration",
                    " Long description",
                    " for the app",
                    "Depends: libc6 (>=2.99), libpython3.10",
                    "Section: utils",
                    "Priority: optional",
                ]
            )
            + "\n"
        )

    package_command.tools.app_tools[
        first_app_deb
    ].app_context.run.assert_called_once_with(
        [
            "dpkg-deb",
            "--build",
            "--root-owner-group",
            "first-app-0.0.1",
        ],
        check=True,
        cwd=bundle_path,
    )

    # The deb was moved into the final location
    package_command.tools.shutil.move.assert_called_once_with(
        bundle_path / "first-app-0.0.1.deb",
        tmp_path / "base_path/dist/first-app_0.0.1-1~somevendor-surprising_wonky.deb",
    )


def test_deb_package_no_long_description(package_command, first_app_deb, tmp_path):
    """A deb app without a long description raises an error."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Delete the long description
    first_app_deb.long_description = None

    # Packaging the app will fail
    with pytest.raises(
        BriefcaseCommandError,
        match=r"App configuration does not define `long_description`. Debian projects require a long description.",
    ):
        package_command.package_app(first_app_deb)

    # The control file won't be written
    assert not (bundle_path / "first-app-0.0.1/DEBIAN/control").exists()


@pytest.mark.parametrize(
    "input, output",
    [
        ["", ""],
        ["one line", "one line"],
        ["first line\nsecond line", "first line\n second line"],
        ["first line\n\nsecond line", "first line\n second line"],
        ["first line\n  \nsecond line", "first line\n second line"],
    ],
)
def test_multiline_long_description(input, output):
    """Multiline debian descriptions are transformed."""
    assert debian_multiline_description(input) == output


def test_deb_package_extra_requirements(package_command, first_app_deb, tmp_path):
    """A deb app can be packaged with extra runtime requirements and configuration
    options."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Add system requirements and other optional settings.
    first_app_deb.system_runtime_requires = ["first", "second (>=1.2.3)"]
    first_app_deb.revision = 42
    first_app_deb.system_section = "Funny stuff"

    # Package the app
    package_command.package_app(first_app_deb)

    # The control file is written
    assert (bundle_path / "first-app-0.0.1/DEBIAN/control").exists()
    with (bundle_path / "first-app-0.0.1/DEBIAN/control").open(encoding="utf-8") as f:
        assert (
            f.read()
            == "\n".join(
                [
                    "Package: first-app",
                    "Version: 0.0.1",
                    "Architecture: wonky",
                    "Maintainer: Megacorp <maintainer@example.com>",
                    "Homepage: https://example.com/first-app",
                    "Description: The first simple app \\ demonstration",
                    " Long description",
                    " for the app",
                    "Depends: libc6 (>=2.99), libpython3.10, first, second (>=1.2.3)",
                    "Section: Funny stuff",
                    "Priority: optional",
                ]
            )
            + "\n"
        )

    package_command.tools.app_tools[
        first_app_deb
    ].app_context.run.assert_called_once_with(
        [
            "dpkg-deb",
            "--build",
            "--root-owner-group",
            "first-app-0.0.1",
        ],
        check=True,
        cwd=bundle_path,
    )

    # The deb was moved into the final location
    package_command.tools.shutil.move.assert_called_once_with(
        bundle_path / "first-app-0.0.1.deb",
        tmp_path / "base_path/dist/first-app_0.0.1-42~somevendor-surprising_wonky.deb",
    )


def test_deb_package_failure(package_command, first_app_deb, tmp_path):
    """If a packaging doesn't succeed, an error is raised."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Mock a packaging failure
    package_command.tools.app_tools[first_app_deb].app_context.run.side_effect = (
        subprocess.CalledProcessError(cmd="dpkg-deb ...", returncode=-1)
    )

    # Package the app
    with pytest.raises(
        BriefcaseCommandError, match=r"Error while building .deb package for first-app."
    ):
        package_command.package_app(first_app_deb)

    # The control file is written
    assert (bundle_path / "first-app-0.0.1/DEBIAN/control").exists()
    with (bundle_path / "first-app-0.0.1/DEBIAN/control").open(encoding="utf-8") as f:
        assert (
            f.read()
            == "\n".join(
                [
                    "Package: first-app",
                    "Version: 0.0.1",
                    "Architecture: wonky",
                    "Maintainer: Megacorp <maintainer@example.com>",
                    "Homepage: https://example.com/first-app",
                    "Description: The first simple app \\ demonstration",
                    " Long description",
                    " for the app",
                    "Depends: libc6 (>=2.99), libpython3.10",
                    "Section: utils",
                    "Priority: optional",
                ]
            )
            + "\n"
        )

    # The call to package was made
    package_command.tools.app_tools[
        first_app_deb
    ].app_context.run.assert_called_once_with(
        [
            "dpkg-deb",
            "--build",
            "--root-owner-group",
            "first-app-0.0.1",
        ],
        check=True,
        cwd=bundle_path,
    )

    # The deb wasn't built, so it wasn't moved.
    package_command.tools.shutil.move.assert_not_called()



import pytest

from briefcase.config import AppConfig
from briefcase.exceptions import UnsupportedPlatform


def test_create_app(tracking_create_command, tmp_path):
    """If the app doesn't already exist, it will be created."""
    tracking_create_command.create_app(tracking_create_command.apps["first"])

    # Input wasn't required by the user
    assert tracking_create_command.input.prompts == []

    # The right sequence of things will be done
    assert tracking_create_command.actions == [
        ("generate", "first"),
        ("support", "first"),
        ("verify-app-template", "first"),
        ("verify-app-tools", "first"),
        ("code", "first", False),
        ("requirements", "first", False),
        ("resources", "first"),
        ("cleanup", "first"),
    ]

    # New app content has been created
    assert (tmp_path / "base_path/build/first/tester/dummy/new").exists()
    # A stub binary has *not* been created
    assert not (tmp_path / "base_path/build/first/tester/dummy/Stub.bin").exists()


def test_create_existing_app_overwrite(tracking_create_command, tmp_path):
    """An existing app can be overwritten if requested."""
    # Answer yes when asked
    tracking_create_command.input.values = ["y"]

    # Generate an app in the location.
    bundle_path = tmp_path / "base_path/build/first/tester/dummy"
    bundle_path.mkdir(parents=True)
    with (bundle_path / "original").open("w", encoding="utf-8") as f:
        f.write("original template!")

    tracking_create_command.create_app(tracking_create_command.apps["first"])

    # Input was required by the user
    assert tracking_create_command.input.prompts == [
        "Application 'first' already exists; overwrite [y/N]? "
    ]

    # The right sequence of things will be done
    assert tracking_create_command.actions == [
        ("generate", "first"),
        ("support", "first"),
        ("verify-app-template", "first"),
        ("verify-app-tools", "first"),
        ("code", "first", False),
        ("requirements", "first", False),
        ("resources", "first"),
        ("cleanup", "first"),
    ]

    # Original content has been deleted
    assert not (bundle_path / "original").exists()

    # New app content has been created
    assert (bundle_path / "new").exists()


def test_create_existing_app_no_overwrite(tracking_create_command, tmp_path):
    """If you say no, the existing app won't be overwritten."""
    # Answer no when asked
    tracking_create_command.input.values = ["n"]

    bundle_path = tmp_path / "base_path/build/first/tester/dummy"
    bundle_path.mkdir(parents=True)
    with (bundle_path / "original").open("w", encoding="utf-8") as f:
        f.write("original template!")
    tracking_create_command.create_app(tracking_create_command.apps["first"])

    # Input was required by the user
    assert tracking_create_command.input.prompts == [
        "Application 'first' already exists; overwrite [y/N]? "
    ]

    # No app creation actions will be performed
    assert tracking_create_command.actions == []

    # Original content still exists
    assert (bundle_path / "original").exists()

    # New app content has not been created
    assert not (bundle_path / "new").exists()


def test_create_existing_app_no_overwrite_default(tracking_create_command, tmp_path):
    """By default, the existing app won't be overwritten."""
    # Answer '' (i.e., just press return) when asked
    tracking_create_command.input.values = [""]

    bundle_path = tmp_path / "base_path/build/first/tester/dummy"
    bundle_path.mkdir(parents=True)
    with (bundle_path / "original").open("w", encoding="utf-8") as f:
        f.write("original template!")

    tracking_create_command.create_app(tracking_create_command.apps["first"])

    # Input was required by the user
    assert tracking_create_command.input.prompts == [
        "Application 'first' already exists; overwrite [y/N]? "
    ]

    # And no actions were necessary
    assert tracking_create_command.actions == []

    # Original content still exists
    assert (bundle_path / "original").exists()

    # New app content has not been created
    assert not (bundle_path / "new").exists()


def test_create_existing_app_input_disabled(tracking_create_command, tmp_path):
    """If input is disabled, fallback to default without get input from user."""
    # Answer '' (i.e., just press return) when asked
    tracking_create_command.input.enabled = False

    bundle_path = tmp_path / "base_path/build/first/tester/dummy"
    bundle_path.mkdir(parents=True)
    with (bundle_path / "original").open("w", encoding="utf-8") as f:
        f.write("original template!")

    tracking_create_command.create_app(tracking_create_command.apps["first"])

    # Input wasn't required by the user
    assert tracking_create_command.input.prompts == []

    # And no actions were necessary
    assert tracking_create_command.actions == []

    # Original content still exists
    assert (bundle_path / "original").exists()

    # New app content has not been created
    assert not (bundle_path / "new").exists()


def test_create_app_not_supported(tracking_create_command, tmp_path):
    """If the supported attribute is false, the command will terminate with an error
    message."""

    with pytest.raises(UnsupportedPlatform):
        tracking_create_command.create_app(
            AppConfig(
                app_name="third",
                bundle="com.example",
                version="0.0.3",
                description="The third simple app",
                sources=["src/third"],
                supported=False,
                license={"file": "LICENSE"},
            )
        )

    # No actions carried out
    assert tracking_create_command.actions == []


def test_create_app_with_stub(tracking_create_command, tmp_path):
    """If an app template defines a stub revision, the stub will be created."""
    # Add an entry to the path index indicating a stub is required
    tracking_create_command._briefcase_toml[tracking_create_command.apps["first"]] = {
        "paths": {"stub_binary_revision": "b1"}
    }

    tracking_create_command.create_app(tracking_create_command.apps["first"])

    # Input wasn't required by the user
    assert tracking_create_command.input.prompts == []

    # The right sequence of things will be done
    assert tracking_create_command.actions == [
        ("generate", "first"),
        ("support", "first"),
        ("stub", "first"),
        ("verify-app-template", "first"),
        ("verify-app-tools", "first"),
        ("code", "first", False),
        ("requirements", "first", False),
        ("resources", "first"),
        ("cleanup", "first"),
    ]

    # New app content and stub binary has been created
    assert (tmp_path / "base_path/build/first/tester/dummy/new").exists()
    assert (tmp_path / "base_path/build/first/tester/dummy/Stub.bin").exists()



import inspect
import os
import subprocess
import time
from unittest.mock import ANY, MagicMock

import pytest

from briefcase.config import AppConfig

from .utils import create_file


def pytest_sessionstart(session):
    """Ensure that tests don't use a color console."""

    os.environ["TERM"] = "dumb"
    os.environ["NO_COLOR"] = "1"
    try:
        del os.environ["FORCE_COLOR"]
    except KeyError:
        pass


# alias so fixtures can still use them
_print = print
_sleep = time.sleep


def monkeypatched_print(*args, **kwargs):
    """Raise an error for calls to print() from briefcase."""
    frame = inspect.currentframe().f_back
    module = inspect.getmodule(frame.f_code)

    # Disallow any use of a bare print() in the briefcase codebase
    if module and module.__name__.startswith("briefcase."):
        pytest.fail(
            "print() should not be invoked directly. Use Log or Console for printing."
        )

    _print(*args, **kwargs)


@pytest.fixture(autouse=True)
def no_print(monkeypatch):
    """Replace builtin ``print()`` for ALL tests."""
    monkeypatch.setattr("builtins.print", monkeypatched_print)


@pytest.fixture
def sleep_zero(monkeypatch):
    """Replace all calls to ``time.sleep(x)`` with ``time.sleep(0)``."""
    monkeypatch.setattr(time, "sleep", MagicMock(wraps=lambda x: _sleep(0)))


@pytest.fixture
def sub_kw():
    """Default keyword arguments for all subprocess calls."""
    return dict(
        text=True,
        encoding=ANY,
        errors="backslashreplace",
    )


@pytest.fixture
def sub_check_output_kw(sub_kw):
    """Default keyword arguments for all subprocess.check_output calls."""
    return {
        **sub_kw,
        **dict(
            stderr=subprocess.STDOUT,
        ),
    }


@pytest.fixture
def sub_stream_kw(sub_kw):
    """Default keyword arguments for all output streaming subprocess calls."""
    return {
        **sub_kw,
        **dict(
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            bufsize=1,
        ),
    }


@pytest.fixture
def first_app_config():
    return AppConfig(
        app_name="first",
        bundle="com.example",
        version="0.0.1",
        description="The first simple app",
        sources=["src/first"],
        license={"file": "LICENSE"},
    )


@pytest.fixture
def first_app_unbuilt(first_app_config, tmp_path):
    # The same fixture as first_app_config,
    # but ensures that the bundle for the app exists
    create_file(
        tmp_path
        / "base_path"
        / "build"
        / "first"
        / "tester"
        / "dummy"
        / "first.bundle",
        "first.bundle",
    )

    return first_app_config


@pytest.fixture
def first_app(first_app_unbuilt, tmp_path):
    # The same fixture as first_app_config,
    # but ensures that the binary for the app exists
    create_file(
        tmp_path / "base_path/build/first/tester/dummy/first.bin",
        "first.bin",
    )

    return first_app_unbuilt



from unittest.mock import MagicMock

import pytest

from briefcase.integrations.cookiecutter import TOMLEscape


@pytest.mark.parametrize(
    "value, expected",
    [
        # Single digit minor
        ("Hello World", "Hello World"),
        ('Hello " World', 'Hello \\" World'),
        ("Hello \\ World", "Hello \\\\ World"),
    ],
)
def test_escape_toml(value, expected):
    env = MagicMock()
    env.filters = {}
    TOMLEscape(env)
    assert env.filters["escape_toml"](value) == expected


@pytest.mark.parametrize(
    "value, expected",
    [
        ("helloworld", "helloworld"),
        ("helloworldı", '"helloworldı"'),
    ],
)
def test_escape_non_ascii(value, expected):
    env = MagicMock()
    env.filters = {}
    TOMLEscape(env)
    assert env.filters["escape_non_ascii"](value) == expected



def test_overrides_are_not_used(convert_command):
    overrides = {"gui_framework": "Toga"}
    out = convert_command.build_gui_context({}, overrides)
    assert out == {"gui_framework": "None"}
    assert overrides == {"gui_framework": "Toga"}



import pytest


@pytest.mark.parametrize(
    "name",
    [
        "helloworld",
        "existing",
    ],
)
def test_valid_app_name(convert_command, name, tmp_path):
    """Test that valid app names are accepted, including pre-existing names."""
    (tmp_path / "existing").mkdir()

    # This is mostly tested by the equivalent test for the `new` command;
    # This test exists for a check that basic functionality still exists,
    # despite the other overrides.
    assert convert_command.validate_app_name(name)



import pytest
from rich.markup import escape

from briefcase.console import InputDisabled
from tests.utils import default_rich_prompt


def test_call_returns_user_input_when_enabled(console):
    """If input wrapper is enabled, call returns user input."""
    value = "abs"
    prompt = "> "
    console.input.return_value = value

    actual_value = console(prompt=prompt)

    assert actual_value == value
    console.input.assert_called_once_with(default_rich_prompt(prompt), markup=True)


def test_call_returns_user_input_when_enabled_with_markup_prompt(console):
    """If input wrapper is enabled, call returns user input with a prompt with existing
    markup."""
    value = "abs"
    prompt = f"[red]{escape('this is prompt with escaped [markup] text')}[/red]"
    console.input.return_value = value

    actual_value = console(prompt=prompt, markup=True)

    assert actual_value == value
    console.input.assert_called_once_with(prompt, markup=True)


def test_call_raise_exception_when_disabled(disabled_console):
    """If input wrapper is disabled, call raise an exception."""
    prompt = "> "

    with pytest.raises(InputDisabled):
        disabled_console(prompt=prompt)
    disabled_console.input.assert_not_called()


def test_call_raise_keyboardinterrupt_for_eoferror(console):
    """Ensure KeyboardInterrupt is raised when users send EOF to an input prompt."""
    console.input.side_effect = EOFError()

    with pytest.raises(KeyboardInterrupt):
        console(prompt="")



import sys
from contextlib import suppress
from pathlib import Path

from briefcase.cmdline import parse_cmdline
from briefcase.console import Console, Log, Printer
from briefcase.exceptions import (
    BriefcaseError,
    BriefcaseTestSuiteFailure,
    BriefcaseWarning,
    HelpText,
)


def main():
    result = 0
    command = None
    printer = Printer()
    console = Console(printer=printer)
    logger = Log(printer=printer)
    try:
        Command, extra_cmdline = parse_cmdline(sys.argv[1:], console=console)
        command = Command(logger=logger, console=console)
        options, overrides = command.parse_options(extra=extra_cmdline)
        command.parse_config(
            Path.cwd() / "pyproject.toml",
            overrides=overrides,
        )
        command(**options)
    except HelpText as e:
        logger.info()
        logger.info(str(e))
        result = e.error_code
    except BriefcaseWarning as w:
        # The case of something that hasn't gone right, but in an
        # acceptable way.
        logger.warning(str(w))
        result = w.error_code
    except BriefcaseTestSuiteFailure as e:
        # Test suite status is logged when the test is executed.
        # Set the return code, but don't log anything else.
        result = e.error_code
    except BriefcaseError as e:
        logger.error()
        logger.error(str(e))
        result = e.error_code
        logger.capture_stacktrace()
    except Exception:
        logger.capture_stacktrace()
        raise
    except KeyboardInterrupt:
        logger.warning()
        logger.warning("Aborted by user.")
        logger.warning()
        result = -42
        if logger.save_log:
            logger.capture_stacktrace()
    finally:
        with suppress(KeyboardInterrupt):
            logger.save_log_to_file(command)

    return result


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



import ast
import re
import subprocess
import sys
from pathlib import Path
from typing import List

from briefcase.commands.create import _is_local_requirement
from briefcase.commands.open import OpenCommand
from briefcase.config import AppConfig
from briefcase.exceptions import BriefcaseCommandError, ParseError

DEFAULT_OUTPUT_FORMAT = "system"

DEBIAN = "debian"
RHEL = "rhel"
ARCH = "arch"
SUSE = "suse"


def parse_freedesktop_os_release(content):
    """Parse the content of an /etc/os-release file.

    Implementation adapted from Example 5 of
    https://www.freedesktop.org/software/systemd/man/os-release.html

    :param content: The text content of the /etc/os-release file.
    :returns: A dictionary of key-value pairs, in the same format returned by
        `platform.freedesktop_os_release()`.
    """
    values = {}
    for line_number, line in enumerate(content.split("\n"), start=1):
        line = line.rstrip()
        if not line or line.startswith("#"):
            continue
        m = re.match(r"([A-Z][A-Z_0-9]+)=(.*)", line)
        if m:
            name, val = m.groups()
            if val and val[0] in "\"'":
                try:
                    val = ast.literal_eval(val)
                except SyntaxError as e:
                    raise ParseError(
                        "Failed to parse output of FreeDesktop os-release file; "
                        f"Line {line_number}: {e}"
                    )
            values[name] = val
        else:
            raise ParseError(
                "Failed to parse output of FreeDesktop os-release file; "
                f"Line {line_number}: {line!r}"
            )

    return values


class LinuxMixin:
    platform = "linux"

    def support_package_url(self, support_revision):
        """The URL of the support package to use for apps of this type.

        Linux builds that use a support package (AppImage, Flatpak) use indygreg's
        Standalone Python to provide system packages. See
        `https://github.com/indygreg/python-build-standalone` for details.

        System packages don't use a support package; this is defined by
        the template, so this method won't be invoked
        """
        python_download_arch = self.tools.host_arch
        # use a 32bit Python if using 32bit Python on 64bit hardware
        if self.tools.is_32bit_python and self.tools.host_arch == "aarch64":
            python_download_arch = "armv7"
        elif self.tools.is_32bit_python and self.tools.host_arch == "x86_64":
            python_download_arch = "i686"

        version, datestamp = support_revision.split("+")
        return (
            "https://github.com/indygreg/python-build-standalone/releases/download/"
            f"{datestamp}/cpython-{support_revision}-{python_download_arch}-unknown-linux-gnu-install_only.tar.gz"
        )

    def vendor_details(self, freedesktop_info):
        """Normalize the identity of the target Linux vendor, version, and base.

        :param freedesktop_info: The parsed content of the FreeDesktop /etc/os-release
            file. This is the same format returned by
            `platform.freedesktop_os_release()`.
        :returns: A tuple of (vendor, version, vendor_base).
        """
        vendor = freedesktop_info["ID"]
        try:
            codename = freedesktop_info["VERSION_CODENAME"]
            if not codename:
                # Fedora *has* a VERSION_CODENAME key, but it is empty.
                # Treat it as missing.
                raise KeyError("VERSION_CODENAME")
        except KeyError:
            try:
                # Arch uses a specific constant in VERSION_ID
                if freedesktop_info["VERSION_ID"] == "TEMPLATE_VERSION_ID":
                    codename = "rolling"
                else:
                    codename = freedesktop_info["VERSION_ID"].split(".")[0]
            except KeyError:
                # Manjaro doesn't have a VERSION_ID key
                codename = "rolling"

        # Process the vendor_base from the vendor.
        id_like = freedesktop_info.get("ID_LIKE", "").split()
        if vendor == DEBIAN or DEBIAN in id_like or "ubuntu" in id_like:
            vendor_base = DEBIAN
        elif vendor == RHEL or vendor == "fedora" or RHEL in id_like:
            vendor_base = RHEL
        elif vendor == ARCH or ARCH in id_like:
            vendor_base = ARCH
        elif vendor == SUSE or SUSE in id_like:
            vendor_base = SUSE
        else:
            vendor_base = None

        return vendor, codename, vendor_base


class LocalRequirementsMixin:  # pragma: no-cover-if-is-windows
    # A mixin that captures the process of compiling requirements that are specified
    # as local file references into sdists, and then installing those requirements
    # from the sdist.

    def local_requirements_path(self, app):
        return self.bundle_path(app) / "_requirements"

    def _install_app_requirements(
        self,
        app: AppConfig,
        requires: List[str],
        app_packages_path: Path,
    ):
        """Install requirements for the app with pip.

        This method pre-compiles any requirement defined using a local path reference
        into an sdist tarball. This will be used when installing under Docker, as local
        file references can't be accessed in the Docker container.

        :param app: The app configuration
        :param requires: The list of requirements to install
        :param app_packages_path: The full path of the app_packages folder into which
            requirements should be installed.
        """
        # If we're re-building requirements, purge any pre-existing local
        # requirements.
        local_requirements_path = self.local_requirements_path(app)
        if local_requirements_path.exists():
            self.tools.shutil.rmtree(local_requirements_path)
        self.tools.os.mkdir(local_requirements_path)

        # Iterate over every requirement, looking for local references
        for requirement in requires:
            if _is_local_requirement(requirement):
                if Path(requirement).is_dir():
                    # Requirement is a filesystem reference
                    # Build an sdist for the local requirement
                    with self.input.wait_bar(f"Building sdist for {requirement}..."):
                        try:
                            self.tools.subprocess.check_output(
                                [
                                    sys.executable,
                                    "-X",
                                    "utf8",
                                    "-m",
                                    "build",
                                    "--sdist",
                                    "--outdir",
                                    local_requirements_path,
                                    requirement,
                                ],
                                encoding="UTF-8",
                            )
                        except subprocess.CalledProcessError as e:
                            raise BriefcaseCommandError(
                                f"Unable to build sdist for {requirement}"
                            ) from e
                else:
                    try:
                        # Requirement is an existing sdist or wheel file.
                        self.tools.shutil.copy(requirement, local_requirements_path)
                    except OSError as e:
                        raise BriefcaseCommandError(
                            f"Unable to find local requirement {requirement}"
                        ) from e

        # Continue with the default app requirement handling.
        return super()._install_app_requirements(
            app,
            requires=requires,
            app_packages_path=app_packages_path,
        )

    def _pip_requires(self, app: AppConfig, requires: List[str]):
        """Convert the requirements list to an .deb project compatible format.

        Any local file requirements are converted into a reference to the file generated
        by _install_app_requirements().

        :param app: The app configuration
        :param requires: The user-specified list of app requirements
        :returns: The final list of requirement arguments to pass to pip
        """
        # Copy all the requirements that are non-local
        final = [
            requirement
            for requirement in super()._pip_requires(app, requires)
            if not _is_local_requirement(requirement)
        ]

        # Add in any local packages.
        # The sort is needed to ensure testing consistency
        for filename in sorted(self.local_requirements_path(app).iterdir()):
            final.append(filename)

        return final


class DockerOpenCommand(OpenCommand):  # pragma: no-cover-if-is-windows
    # A command that redirects Open to an interactive shell in the container
    # if Docker is being used. Relies on the final command to provide
    # verification that Docker is available, and verify the app context.

    def _open_app(self, app: AppConfig):
        # If we're using Docker, open an interactive shell in the container.
        # Rely on the default CMD statement in the image's Dockerfile to
        # define a default shell.
        if self.use_docker:
            self.tools[app].app_context.run([], interactive=True)
        else:
            super()._open_app(app)
